#include <gtk/gtk.h>
#define __USE_BSD
#include <math.h>
#include <stdbool.h>
#include "graph.h"

int gnam; // should be deleted

typedef struct{
  double *xdata;
  double *ydata;
  int length;
  cairo_pattern_t *pattern;
  gboolean isVisible;
} line_data;

typedef struct{
  line_data *lines;
  int line_num;
  double xmin, xmax;
  double ymin, ymax;
  gchar *xlabel;
  gchar *ylabel;
  gboolean is_logscale_x, is_logscale_y;
  gboolean is_logscale_positive_x, is_logscale_positive_y;
  gboolean show_numbers;
} graph_data;

void setLogscale(GtkWidget *canvas, gboolean is_logscale, gboolean is_logscale_positive, gboolean isX){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if(isX){
    data->is_logscale_x = is_logscale;
    data->is_logscale_positive_x = is_logscale_positive;
  }else{
    data->is_logscale_y = is_logscale;
    data->is_logscale_positive_y = is_logscale_positive;
  }
}

void getLogscale(GtkWidget *canvas, gboolean *is_logscale, gboolean *is_logscale_positive, gboolean isX){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if(isX){
    *is_logscale = data->is_logscale_x;
    *is_logscale_positive = data->is_logscale_positive_x;
  }else{
    *is_logscale = data->is_logscale_y;
    *is_logscale_positive = data->is_logscale_positive_y;
  }
}

double linear(double value, double val_min, double val_max, int win_min, int win_max){
  double scale;
  scale = (win_max - win_min)/(val_max - val_min);
  return win_min + (value  - val_min)* scale;
}

double logarithm(double value, double val_min, double val_max, int win_min, int win_max){
  double lval_min, lval_max, scale;
  lval_min = log10(val_min);
  lval_max = log10(val_max);
  scale = (win_max - win_min)/(lval_max - lval_min);
  return win_min + (log10(value)  - lval_min)* scale;
}
static gboolean cb_expose_event(GtkWidget *canvas, GdkEventExpose *event, graph_data *data){
  GdkWindow *drawable = canvas->window;
  cairo_t *cr;
  const double axis_width  = 3;
  double xscale, yscale, xoffset, yoffset, xwidth, yheight, xmin, xmax, ymin, ymax;
  gint width, height;

  cr = gdk_cairo_create(drawable);

  cairo_set_line_width(cr, axis_width);
  //gtk_widget_get_size_request(canvas, &width, &height);
  //gtk_widget_size_request(canvas,&requisition);
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);
  xmin = data->xmin;
  xmax = data->xmax;
  if(data->is_logscale_x){
    if(data->is_logscale_positive_x){
      if(xmax <= 0) xmax = 1;
      if(xmin <= 0) xmin = xmax/10;
    }else{
      if(xmin >= 0) xmin = -1;
      if(xmax >= 0) xmax = xmin/10;
    }
  }
  ymin = data->ymin;
  ymax = data->ymax;
  if(data->is_logscale_y){
    if(data->is_logscale_positive_y){
      if(ymax <= 0) ymax = 1;
      if(ymin <= 0) ymin = ymax/10;
    }else{
      if(ymin >= 0) ymin = -1;
      if(ymax >= 0) ymax = ymin/10;
    }
  }
  
  xwidth = xmax - xmin;
  yheight = ymax - ymin;
  xscale = 0.7*width/xwidth;
  yscale = 0.7*height/yheight;
  xoffset = 0.15*width;
  yoffset = 0.85*height;
  // draw background
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_rectangle(cr, width*0.01, height*0.01, width*0.98, height*0.98);
  cairo_fill(cr);
  cairo_stroke(cr);

  // draw axes
  cairo_set_source_rgb(cr, 0, 1, 0);
  cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
  cairo_move_to(cr, xoffset, yoffset);
  if(data->is_logscale_x) cairo_line_to(cr, xoffset+log10(fabs(xmax))*xscale, yoffset);
  else cairo_line_to(cr, xoffset+xwidth*xscale, yoffset);
  cairo_move_to(cr, xoffset, yoffset);
  if(data->is_logscale_y) cairo_line_to(cr, xoffset, log10(fabs(ymax))*yscale);
  else cairo_line_to(cr, xoffset, yoffset - yheight*yscale);
  cairo_stroke(cr);

  // draw sub-axis
  double (*xfunc)(double, double, double, int, int);
  double (*yfunc)(double, double, double, int, int);
  if(data->is_logscale_x) xfunc = logarithm;
  else xfunc = linear;
  if(data->is_logscale_y) yfunc = logarithm;
  else yfunc = linear;

  if(data->is_logscale_x){
  }else{
    double powerx,powery, xsubwidth, ysubheight;
    double temp;
    double xdash[] = {10,10}, ydash[] = {10,10};
    double fontsize;
    cairo_set_dash(cr, xdash, 2,0);
    fontsize = height*0.05;
    fontsize = fontsize > 2 ? fontsize : 2;
    cairo_set_font_size(cr,fontsize);
    temp = log10(xwidth/5);
    powerx = floor(temp);
    xsubwidth = pow(10,temp - powerx);
    if(xsubwidth <= 1.6) xsubwidth = 1;
    else if(xsubwidth <= 2.4) xsubwidth = 2;
    else if(xsubwidth <= 4) xsubwidth = 2.5;
    else if (xsubwidth <= 8) xsubwidth = 5;
    else xsubwidth = 10;
    xsubwidth = xsubwidth*pow(10,powerx);
    /*
    if(fontsize/2*8 > xsubwidth*xscale){
      cairo_matrix_t tm;
      cairo_get_font_matrix(cr, &tm);
      tm.xx = xsubwidth*xscale/8*2*0.8;
      cairo_set_font_matrix(cr, &tm);
    }*/
    if(fontsize/2*8 > xsubwidth*xscale) fontsize = xsubwidth*xscale/8*2*0.8;
    cairo_set_font_size(cr,fontsize);
    temp = ceil(xmin/xsubwidth)*xsubwidth;
    while(temp <= xmax){
      char text[21];
      cairo_move_to(cr, xoffset+xscale*(temp - xmin), yoffset);
      cairo_line_to(cr, xoffset+xscale*(temp - xmin), yoffset-yscale*yheight);
      cairo_move_to(cr, xoffset+xscale*(temp - xmin) - fontsize*2, yoffset + fontsize);
      snprintf(text,20,"%.2e", temp);
      cairo_show_text(cr, text);
      temp += xsubwidth;
    }
    cairo_stroke(cr);
  }
  if(data->is_logscale_y){
    double ydash1[] = {10,10}, ydash2[] = {5,5};
    double temp;
    temp = pow(10, ceil(log10(ymin)));
    while(temp <  ymax){
      cairo_set_line_width(cr, axis_width);
      cairo_set_dash(cr, ydash1, 2,0);
      cairo_move_to(cr, xoffset, (*yfunc)(temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_line_to(cr, (*xfunc)(xmax, xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_stroke(cr);
      cairo_set_line_width(cr, 1);
      cairo_set_dash(cr, ydash2, 2,0);
      cairo_move_to(cr, xoffset, (*yfunc)(2*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_line_to(cr, (*xfunc)(xmax, xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(2*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_move_to(cr, xoffset, (*yfunc)(4*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_line_to(cr, (*xfunc)(xmax, xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(4*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_move_to(cr, xoffset, (*yfunc)(8*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_line_to(cr, (*xfunc)(xmax, xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(8*temp, ymin, ymax, yoffset, yoffset - yheight*yscale));
      cairo_stroke(cr);
      temp *= 10;
    }
    cairo_stroke(cr);
  }else{
    double powerx,powery, xsubwidth, ysubheight;
    double temp;
    double xdash[] = {10,10}, ydash[] = {10,10};
    double fontsize;
    fontsize = width*0.02;
    fontsize = fontsize > 2 ? fontsize : 2;
    cairo_set_font_size(cr,fontsize);
    cairo_set_dash(cr, xdash, 2,0);
    temp = log10(yheight/5);
    powery = floor(temp);
    ysubheight = pow(10,temp - powery);
    if(ysubheight <= 1.6) ysubheight = 1;
    else if(ysubheight <= 2.4) ysubheight = 2;
    else if(ysubheight <= 4) ysubheight = 2.5;
    else if (ysubheight <= 8) ysubheight = 5;
    else ysubheight = 10;
    ysubheight = ysubheight*pow(10,powery);
    temp = ceil(data->ymin/ysubheight)*ysubheight;
    while(temp <= data->ymax){
      char text[21];
      cairo_move_to(cr, xoffset, yoffset-(temp - data->ymin)*yscale);
      cairo_line_to(cr, xoffset+xscale*xwidth, yoffset-(temp- data->ymin)*yscale);
      cairo_move_to(cr, xoffset - 4*fontsize, yoffset-(temp - data->ymin)*yscale);
      snprintf(text,20,"%.2e", temp);
      cairo_show_text(cr, text);
      temp += ysubheight;
    }
    cairo_stroke(cr);
    cairo_set_dash(cr, NULL, 0, 0);
  }

   cairo_set_line_width(cr, axis_width);
  // set labels
  if(data->xlabel != NULL){
    cairo_move_to(cr, width/2, height*0.96);
    cairo_show_text(cr,data->xlabel);
  }
  if(data->ylabel != NULL){
    cairo_save(cr);
    cairo_move_to(cr, width*0.03, height/2);
    cairo_rotate(cr, -M_PI/2);
    cairo_show_text(cr,data->ylabel);
    cairo_restore(cr);
  }

  // draw data
  for(int i =0;i < data->line_num;i++){
    line_data *line = data->lines+i;
    if(line->isVisible){
      cairo_set_source(cr,line->pattern);
      cairo_move_to(cr, (*xfunc)(*(line->xdata), xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(*(line->ydata), ymin, ymax, yoffset, yoffset - yheight*yscale));
      for(int j = 1; j < line->length;j++){
        cairo_line_to(cr, (*xfunc)(*(line->xdata + j), xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(*(line->ydata + j), ymin, ymax, yoffset, yoffset - yheight*yscale));
      }
      cairo_stroke(cr);
    }
  }

  // set numbers
  if(data->show_numbers){
    GRand *grand;
    grand = g_rand_new_with_seed(0);
    for(int i = 0; i < data->line_num; i++){
      line_data *line = data->lines+i;
      if(line->isVisible){
        gint32 j;
        gchar text[4];
        j = g_random_int_range(0, line->length);
        cairo_set_source(cr, line->pattern);
        cairo_move_to(cr, (*xfunc)(*(line->xdata + j), xmin, xmax, xoffset, xoffset+xwidth*xscale), (*yfunc)(*(line->ydata + j), ymin, ymax, yoffset, yoffset - yheight*yscale));
        snprintf(text,3,"%d", i);
        cairo_show_text(cr, text);
        cairo_stroke(cr);
      }
    }
    g_rand_free(grand);
  }

  cairo_destroy(cr);
  return FALSE;
}

void show_numbers(GtkWidget *canvas, gboolean show){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  data->show_numbers = show;
}

static void cb_destroy(GtkWidget *canvas, gpointer p){
  
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return;
  for(int i = 0; i < data->line_num; i++){
    g_free((data->lines + i)->xdata);
    g_free((data->lines + i)->ydata);
  }
  g_free(data->lines);
  g_free(data);
  g_message("graph is destroyed");
}


static gboolean cb_button_press_event(GtkWidget *canvas, GdkEventButton *event, graph_data *data){
  // calculate the position
  gdouble x, y;
  x = event->x;
  y = event->y;
  GdkWindow *drawable = canvas->window;
  double xscale, yscale, xoffset, yoffset, xwidth, yheight, xmin, xmax, ymin, ymax;
  gint width, height;
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);
  xmin = data->xmin;
  xmax = data->xmax;
  // not for log
  if(data->is_logscale_x){
    if(data->is_logscale_positive_x){
      if(xmax <= 0) xmax = 1;
      if(xmin <= 0) xmin = xmax/10;
    }else{
      if(xmin >= 0) xmin = -1;
      if(xmax >= 0) xmax = xmin/10;
    }
  }
  ymin = data->ymin;
  ymax = data->ymax;
  if(data->is_logscale_y){
    if(data->is_logscale_positive_y){
      if(ymax <= 0) ymax = 1;
      if(ymin <= 0) ymin = ymax/10;
    }else{
      if(ymin >= 0) ymin = -1;
      if(ymax >= 0) ymax = ymin/10;
    }
  }
  
  xwidth = xmax - xmin;
  yheight = ymax - ymin;
  xscale = 0.7*width/xwidth;
  yscale = 0.7*height/yheight;
  xoffset = 0.15*width;
  yoffset = 0.85*height;
  x = (x - xoffset)/xscale + xmin;
  y = (yoffset - y)/yscale + ymin;

  // find the line
  double distance = DBL_MAX;
  int index = -1;
  for(int i =0;i < data->line_num;i++){
    double d = DBL_MAX;
    line_data *line = data->lines+i;
    if(line->isVisible){
      for(int j = 0; j < line->length;j++){
        double dd;
        dd = sqrt(pow(*(line->xdata +j) - x, 2) + pow(*(line->ydata + j) - y,2));
        d = dd < d ? dd : d;
      }
      if(distance > d){
        distance = d;
        index = i;
      }
    }
  }

  //
  if(index >= 0){
    gnam = index;
  }
  return FALSE;
}

void hide_line(GtkWidget *canvas, gint lnum){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL || data->line_num <= lnum) return;
  (data->lines + lnum)->isVisible = FALSE;
}

gboolean graph_is_visible(GtkWidget *canvas, gint lnum){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL || data->line_num <= lnum) return FALSE;
  return (data->lines + lnum)->isVisible;
}


void show_line(GtkWidget *canvas, gint lnum){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL || data->line_num <= lnum) return;
  (data->lines + lnum)->isVisible = TRUE;
}

gint append_line(GtkWidget *canvas, double *xdata, double *ydata, int num, double r, double g, double b){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return -1;
  line_data *temp;
  data->line_num++;
  int lnum = data->line_num -1;
  temp = (line_data *) g_renew(line_data, data->lines, data->line_num);
  data->lines = temp;
  (data->lines + lnum)->xdata = (gdouble *) g_memdup(xdata, sizeof(gdouble) * num);
  (data->lines + lnum)->ydata = (gdouble *) g_memdup(ydata, sizeof(gdouble) * num);;
  (data->lines + lnum)->length = num;
  (data->lines + lnum)->pattern = cairo_pattern_create_rgb(r,g,b);
  (data->lines + lnum)->isVisible = TRUE;
  return lnum;
}

void change_color(GtkWidget *canvas, int num, double r, double g, double b){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL || data->line_num < num) return;
  line_data *temp;
  (data->lines + num)->pattern = cairo_pattern_create_rgb(r,g,b);
  return;
}

void clear_line(GtkWidget *canvas){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if(data == NULL) return ;
  for(int i = 0; i < data->line_num; i++){
    g_free((data->lines + i)->xdata);
    g_free((data->lines + i)->ydata);
  }
  g_free(data->lines);
  data->lines = NULL;
  data->line_num = 0;
}

void set_xlabel(GtkWidget *canvas, gchar *label){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return;
  g_free(data->xlabel);
  data->xlabel = g_strdup(label);
}

void set_ylabel(GtkWidget *canvas, gchar *label){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return;
  g_free(data->ylabel);
  data->ylabel = g_strdup(label);
}

void set_xrange(GtkWidget *canvas, gdouble xmin, gdouble xmax){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return;
  data->xmin = xmin;
  data->xmax = xmax;
}

void set_yrange(GtkWidget *canvas, gdouble ymin, gdouble ymax){
  graph_data *data = (graph_data*) g_object_get_data(G_OBJECT(canvas), "graph_data");
  if (data == NULL) return;
  data->ymin = ymin;
  data->ymax = ymax;
}

GtkWidget *my_graph(void){
  GtkWidget *canvas;
  graph_data *data;

  canvas = gtk_drawing_area_new();
  data = g_new0(graph_data, 1);
  data->xmin = 0;
  data->ymin = 0;
  data->xmax = 1;
  data->ymax = 1;
  data->line_num = 0;
  data->xlabel = NULL;
  data->ylabel = NULL;
  data->lines = NULL;
  data->line_num = 0;
  data->is_logscale_x = FALSE;
  data->is_logscale_y = FALSE;
  data->is_logscale_positive_x = TRUE;
  data->is_logscale_positive_y = TRUE;
  data->show_numbers = FALSE;
  g_object_set_data(G_OBJECT(canvas), "graph_data", data);
  g_signal_connect(G_OBJECT(canvas), "expose-event", G_CALLBACK(cb_expose_event), data);
  gtk_widget_add_events(canvas, GDK_BUTTON_PRESS_MASK);
  g_signal_connect(G_OBJECT(canvas), "button-press-event", G_CALLBACK(cb_button_press_event), data);
  g_signal_connect(G_OBJECT(canvas), "destroy", G_CALLBACK(cb_destroy), data);
  return canvas;
}
