#include <gtk/gtk.h>
#include <stdlib.h>
#include <complex.h>
#define __USE_BSD
#include <math.h>
#include "graph.h"
#include "state.h"
#include <gtdispc.h>

/*
  memo
  When you change the species of particle, your modification is not saved.
*/

static gulong id_changed;

static GtkWidget *B_cv, *B_wc, *B_theta;
static GtkWidget *B_q, *B_qm, *B_drift, *B_wpd, *B_para, *B_perp, *B_ppp, *B_loss0, *B_bfac;
static GtkWidget *B_frmin, *B_frmax, *B_fimin, *B_fimax, *B_k;
static GtkWidget *B_name, *Combo;

static int istate = 0;

static void cb_changed(GtkWidget *combo, gpointer user_data);
static void override_val(void){
  const gchar *text;
  gchar text2[25];

  text = gtk_entry_get_text(GTK_ENTRY(B_q));
  *(an + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_qm));
  *(qm + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_wpd));
  *(wpd + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_para));
  *(path + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_perp));
  *(peth + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_drift));
  *(spa + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_ppp));
  *(ppp + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_loss0));
  *(loss0 + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_bfac));
  *(bfac + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_name));
  snprintf(*(sname + istate), 24, "%s", text);
  g_signal_handler_disconnect(G_OBJECT(Combo), id_changed);
  gtk_combo_box_remove_text(GTK_COMBO_BOX(Combo), istate);
  snprintf(text2, 24, "%d:%s", istate+1, *(sname + istate));
  gtk_combo_box_insert_text(GTK_COMBO_BOX(Combo), istate, text2);
  id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);
}

static void update_val1(void){
  const gchar *text;

  text = gtk_entry_get_text(GTK_ENTRY(B_cv));
  cv = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_wc));
  wc = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_theta));
  theta = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_frmin));
  cxi = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_frmax));
  cxa = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_fimin));
  cyi = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_fimax));
  cya = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_k));
  xcs = strtod(text, NULL);
}

static void destroy(GtkWidget *w, gpointer p){
  g_message("Program closed");
  gtk_main_quit();
}

static gint parse_data(gchar* filename){
  FILE *fp=NULL;
  fp = g_fopen(filename, "r");
  int num_before;
  num_before = num;
  if(fp){
    int result;
    result = fscanf(fp, "%d %lf %lf\n", &num, &wc, &cv);
    if(result != 3 || num <= 0 || wc == 0 || cv <= 0) goto file_is_dead;
    result = fscanf(fp, "%d %d %d\n", &nx, &ny, &nk);
    if(result != 3 || nx <= 0 || ny <= 0 || nk <= 0) goto file_is_dead;
    result = fscanf(fp, "%lf\n", &theta);
    if(result != 1 || theta > 90 || theta < 0) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &xcs, &xcl);
    if(result != 2 || xcs < 0 || xcl < xcs) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &cxi, &cxa);
    if(result != 2 ||  cxa < cxi) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &cyi, &cya);
    if(result != 2 ||  cya < cyi) goto file_is_dead;

    xcs = (xcs+xcl)/2;

    initialize_parameters(num);
    for(int i = 0; i < num_before; i++) free(*(sname + i));
    free(sname);
    sname = (char **) malloc(sizeof(char *) * num);
    //if(!an || !wpd || !qm || !peth || !path || !spa || !loss0 || !bfac || !ppp) goto malloc_error;
    for(int i = 0; i < num; i++){
      *(sname + i) = (char *) malloc(sizeof(char) * 21);
      fscanf(fp, "%20s\n", *(sname + i));
      result = fscanf(fp, "%lf %lf %lf\n", an+i, wpd+i, qm+i);
      if(result != 3) goto file_is_dead;
      result = fscanf(fp, "%lf %lf %lf\n", peth+i, path+i, spa+i);
      if(result != 3) goto file_is_dead;
      result = fscanf(fp, "%d %lf %lf\n", loss0+i, bfac+i, ppp+i);
      if(result != 3) goto file_is_dead;
    }
    char text[25];
    snprintf(text,20,"%e", wc);
    gtk_entry_set_text(GTK_ENTRY(B_wc), text);
    snprintf(text,20,"%e", cv);
    gtk_entry_set_text(GTK_ENTRY(B_cv), text);
    snprintf(text,20,"%e", theta);
    gtk_entry_set_text(GTK_ENTRY(B_theta), text);
    snprintf(text,20,"%e", *an);
    gtk_entry_set_text(GTK_ENTRY(B_q), text);
    snprintf(text,20,"%e", *qm);
    gtk_entry_set_text(GTK_ENTRY(B_qm), text);
    snprintf(text,20,"%e", *wpd);
    gtk_entry_set_text(GTK_ENTRY(B_wpd), text);
    snprintf(text,20,"%e", *spa);
    gtk_entry_set_text(GTK_ENTRY(B_drift), text);
    snprintf(text,20,"%e", *path);
    gtk_entry_set_text(GTK_ENTRY(B_para), text);
    snprintf(text,20,"%e", *peth);
    gtk_entry_set_text(GTK_ENTRY(B_perp), text);
    snprintf(text,20,"%d", *loss0);
    gtk_entry_set_text(GTK_ENTRY(B_loss0), text);
    snprintf(text,20,"%e", *bfac);
    gtk_entry_set_text(GTK_ENTRY(B_bfac), text);
    snprintf(text,20,"%e", *ppp);
    gtk_entry_set_text(GTK_ENTRY(B_ppp), text);
    snprintf(text,20,"%e", cxi);
    gtk_entry_set_text(GTK_ENTRY(B_frmin), text);
    snprintf(text,20,"%e", cxa);
    gtk_entry_set_text(GTK_ENTRY(B_frmax), text);
    snprintf(text,20,"%e", cyi);
    gtk_entry_set_text(GTK_ENTRY(B_fimin), text);
    snprintf(text,20,"%e", cya);
    gtk_entry_set_text(GTK_ENTRY(B_fimax), text);
    snprintf(text,20,"%e", xcs);
    gtk_entry_set_text(GTK_ENTRY(B_k), text);
    gtk_entry_set_text(GTK_ENTRY(B_name), *sname);

    g_signal_handler_disconnect(G_OBJECT(Combo), id_changed);
    for(int i = 0; i <= num_before; i++){
      gtk_combo_box_remove_text(GTK_COMBO_BOX(Combo), 0);
    }
    for(int i = 0; i < num; i++){
      gchar text[25];
      snprintf(text, 24, "%d:%s", i+1, *(sname + i));
      gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), text);
    }
    gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "Add");
    gtk_combo_box_set_active(GTK_COMBO_BOX(Combo), 0);
    id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);

    istate = 0;
  }else{
    return 1;
  }
  return 0;
  file_is_dead:
  // here should be implemented
    return 2;
}

static void cb_open(GtkWidget *w, GtkWindow *window){
  GtkWidget *dialog;
  gint response;
  GtkFileFilter *filter, *nofilter;
  dialog = gtk_file_chooser_dialog_new("Restore parameters", GTK_WINDOW(window), GTK_FILE_CHOOSER_ACTION_OPEN,\
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  filter = gtk_file_filter_new();
  nofilter = gtk_file_filter_new();
  gtk_file_filter_set_name(filter, "Parameter Files");
  gtk_file_filter_set_name(nofilter, "ALL Files");
  gtk_file_filter_add_pattern(filter, "*.prm");
  gtk_file_filter_add_pattern(nofilter, "*");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), nofilter);
  gtk_widget_show_all(dialog);
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  if ( response == GTK_RESPONSE_ACCEPT){
    int result;
    gchar *filename, *basename, *title;
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    result = parse_data(filename);
    basename = g_path_get_basename(filename);
    title = g_strdup_printf("KUPDAP: %s", basename);
    gtk_window_set_title(window, title);
    g_free(title);
    g_free(basename);
    g_free(filename);
    if(result == 1) g_message("fail to load");
    else if(result == 2) g_message("fail to parse");
  }
  gtk_widget_destroy(dialog);
}

reserved_state init_state;

static void copy_state(){
  init_state.cv = cv;
  init_state.theta = theta;
  init_state.wc = wc;
  init_state.val = val;
  init_state.b = b;
  init_state.num = num;

  if(init_state.an != NULL){
    free(init_state.an);
    free(init_state.loss0);
  }
  init_state.an = (double *) malloc(sizeof(double)*num*10);
  init_state.loss0 = (int *) malloc(sizeof(int)*num);
  init_state.wpd  = init_state.an+1*num;
  init_state.qm   = init_state.an+2*num;
  init_state.peth = init_state.an+3*num;
  init_state.path = init_state.an+4*num;
  init_state.spa  = init_state.an+5*num;
  init_state.bfac = init_state.an+6*num;
  init_state.ppp  = init_state.an+7*num;
  init_state.rn   = init_state.an+8*num;
  init_state.wp   = init_state.an+9*num;
  for(int i = 0; i < num; i++){
    *(init_state.an+i)   = *(an+i);
    *(init_state.wpd+i)  = *(wpd+i);
    *(init_state.qm+i)   = *(qm+i);
    *(init_state.peth+i) = *(peth+i);
    *(init_state.path+i) = *(path+i);
    *(init_state.spa+i)  = *(spa+i);
    *(init_state.bfac+i) = *(bfac+i);
    *(init_state.ppp+i)  = *(ppp+i);
    *(init_state.rn+i)   = *(rn+i);
    *(init_state.wp+i)   = *(wp+i);
    *(init_state.loss0+i)= *(loss0+i);
  }

  init_state.kmin = xcs;
  init_state.kmax = xcl;
  init_state.wmin = cxi;
  init_state.wmax = cxa;
  init_state.rmin = cyi;
  init_state.rmax = cya;
  
}

static void cb_main2(GtkWidget *w, gpointer user_data){
  double thr, yr, yi;
  double h_val[200*200];
  double x1[200*200], x2[200*200], y1[200*200], y2[200*200];
  update_val1();
  if(istate >= 0)override_val();
  calculate_unit();
  get_basic_parameters();

  // calculate the table of H
  thr = theta/180*M_PI;
  yr = xcs*sin(thr);
  yi = xcs*cos(thr);
  bessmd(yr);
  for(int iw = 0; iw < 200; iw++){
    for(int ig = 0; ig < 200; ig++){
      complex f;
      x1[iw*200 + ig] = (cxa - cxi)/200*iw + cxi;
      x2[iw*200 + ig] = (cxa - cxi)/200*(iw+1) + cxi;
      y1[iw*200 + ig] = (cya - cyi)/200*ig + cyi;
      y2[iw*200 + ig] = (cya - cyi)/200*(ig+1) + cyi;
      f= (cxa - cxi)/200*(iw+0.5) + cxi + I*( (cya - cyi)/200*(ig+0.5) + cyi);
      h_val[iw*200 + ig] = log10(cabs(htodh(f, yr, yi)));
    }
  }

  // create a color contour
  GtkWidget *win;
  win = main2_new(x1, y1, x2, y2, h_val, 200*200);
  gtk_widget_show_all(win);
}

static void cb_changed(GtkWidget *combo, gpointer user_data){
  gchar *text;
  text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(combo));
  if(!g_strcmp0("Add", text)){
    if(istate >= 0)override_val();
    istate = -1;
    // not implemented yet
  }else{
    gint itext;
    gchar t[25];
    if(istate >= 0)override_val();
    itext = atoi(text) - 1;
    snprintf(t,20,"%e", *(an+itext));
    gtk_entry_set_text(GTK_ENTRY(B_q), t);
    snprintf(t,20,"%e", *(qm+itext));
    gtk_entry_set_text(GTK_ENTRY(B_qm), t);
    snprintf(t,20,"%e", *(wpd+itext));
    gtk_entry_set_text(GTK_ENTRY(B_wpd), t);
    snprintf(t,20,"%e", *(spa+itext));
    gtk_entry_set_text(GTK_ENTRY(B_drift), t);
    snprintf(t,20,"%e", *(path+itext));
    gtk_entry_set_text(GTK_ENTRY(B_para), t);
    snprintf(t,20,"%e", *(peth+itext));
    gtk_entry_set_text(GTK_ENTRY(B_perp), t);
    snprintf(t,20,"%d", *(loss0+itext));
    gtk_entry_set_text(GTK_ENTRY(B_loss0), t);
    snprintf(t,20,"%e", *(bfac+itext));
    gtk_entry_set_text(GTK_ENTRY(B_bfac), t);
    snprintf(t,20,"%e", *(ppp+itext));
    gtk_entry_set_text(GTK_ENTRY(B_ppp), t);
    snprintf(t,24,"%s", *(sname + itext));
    gtk_entry_set_text(GTK_ENTRY(B_name), t);
    
    istate = itext;
  }
  g_free(text);
}

int main(int argc, char *argv[]){
  GtkWidget *window, *vbox;
  PangoFontDescription *p;
  
  p = pango_font_description_from_string("Sans 18");
  gtk_init(&argc, &argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "KUPDAP");
  g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(destroy), NULL);

  vbox = gtk_vbox_new(FALSE, 3);
  gtk_container_add(GTK_CONTAINER(window), vbox);

  init_state.an = NULL;
  init_state.loss0 = NULL;
  {
    GtkWidget *frame, *e, *hbox, *label;
    GdkColor color;
    frame = gtk_frame_new("GENERAL");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#FF3366", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), frame);
    e = gtk_event_box_new();
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_container_add(GTK_CONTAINER(frame), e);
    hbox = gtk_hbox_new(TRUE, 3);
    gtk_container_add(GTK_CONTAINER(e), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox),gtk_label_new("cv"));
    B_cv = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_cv);
    gtk_box_pack_start_defaults(GTK_BOX(hbox),gtk_label_new("wc"));
    B_wc = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_wc);
    gtk_box_pack_start_defaults(GTK_BOX(hbox),gtk_label_new("theta"));
    B_theta = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_theta);
  }

  {
    GtkWidget *frame, *e, *vvbox, *hbox, *label;
    GdkColor color;
    frame = gtk_frame_new("PARTICLE");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#3333FF", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), frame);
    e = gtk_event_box_new();
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_container_add(GTK_CONTAINER(frame), e);
    vvbox = gtk_vbox_new(FALSE, 3);
    gtk_container_add(GTK_CONTAINER(e), vvbox);
    {
      hbox = gtk_hbox_new(TRUE, 3);
      gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
      gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("name"));
      B_name = gtk_entry_new();
      gtk_box_pack_start_defaults(GTK_BOX(hbox), B_name);
      gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("species"));
      Combo = gtk_combo_box_new_text();
      gtk_box_pack_start_defaults(GTK_BOX(hbox), Combo);
      gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "1");
      num = 1;
      gtk_combo_box_set_active(GTK_COMBO_BOX(Combo), 0);
      gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "Add");
      id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);
    }

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("q"));
    B_q = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_q);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("q/m"));
    B_qm = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_qm);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("wpd"));
    B_wpd = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_wpd);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("drift"));
    B_drift = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_drift);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("v_perp"));
    B_perp = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_perp);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("v_para"));
    B_para = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_para);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("loss0"));
    B_loss0 = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_loss0);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("bfac"));
    B_bfac = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_bfac);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("ppp"));
    B_ppp = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_ppp);
  }
  {
    GtkWidget *frame, *e, *vvbox, *hbox, *label;
    GdkColor color;
    frame = gtk_frame_new("RANGE");
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#33FF33", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), frame);
    e = gtk_event_box_new();
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_container_add(GTK_CONTAINER(frame), e);
    
    vvbox = gtk_vbox_new(FALSE, 3);
    gtk_container_add(GTK_CONTAINER(e), vvbox);
    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("frmin"));
    B_frmin = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_frmin);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("frmax"));
    B_frmax = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_frmax);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("fimin"));
    B_fimin = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_fimin);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("fimax"));
    B_fimax = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_fimax);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("k"));
    B_k = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_k);
  }
  {
    GtkWidget *hbox, *button;
    hbox = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
    button = gtk_button_new_with_label("Load");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_open), window);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), button);
    button = gtk_button_new_with_label("Calculate");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_main2), window);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), button);

  }

  sname = (char **) malloc(sizeof(char *) * num);
  *(sname) = (char *) malloc(sizeof(char) * 21);

  gtk_widget_show_all(window);
  gtk_main();
}
