#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include <gsl/gsl_sf_bessel.h>

#include "gtdispc.h"


/*
  This function calculates be, dbe, benn and maxb;
  *(be + i*NUM_BE + j) = scaled modified bessel function of the first kind of the j-th order = e^x Ij(x)
  dbe is the derivative of be
  *(benn + i*NUM_BE + j) = e^x Ij(x)/x/j/j
  maxb is the maximum number of j

  x = lambda = Tperp / m *(kperp/Omega)^2

  return
  0 ... success
  1 ... some error has occurred
*/


double *be = 0;
double *dbe = 0;
double *benn = 0;
int *maxb = 0;


int bessmd(double yx)
{
  /* memory allocation */
  if(be) free(be);
  if(dbe) free(dbe);
  if(benn) free(benn);
  if(maxb) free(maxb);

  be = (double *) calloc(NUM_BE*2*num, sizeof(double));
  dbe = (double *) calloc(NUM_BE*2*num, sizeof(double));
  benn = (double *) calloc(NUM_BE*2*num, sizeof(double));
  maxb = (int *) calloc(2*num, sizeof(int));

  if(be == NULL || dbe == NULL || benn == NULL || maxb == NULL)
  {
    printf("malloc failure\n");
    free(be);
    free(dbe);
    free(benn);
    free(maxb);
    return 1;
  }

  /* when yx is close to 0*/
  if(yx < unit)
  {
    for(int i = 0; i<num; i++)
    {
      int j = num + i;
      *(maxb + i) = 3;
      *(maxb + j) = 3;
      *(be + i*NUM_BE) = 1;
      *(be + j*NUM_BE) = 1;
      *(dbe + i*NUM_BE) = -1;
      *(dbe + j*NUM_BE) = -1;
      *(benn + i*NUM_BE + 1) = 0.5;
      *(benn + j*NUM_BE + 1) = 0.5;
      *(dbe + i*NUM_BE + 1) = 0.5;
      *(dbe + j*NUM_BE + 1) = 0.5;
    }
  }else
  {
  /* calculation */
    for(int i = 0; i<num; i++) // i-th particle. (i+num) means the value for loss-cone distribution
    {
      double x;      // lambda = Tperp / m *(kperp/Omega)^2
      bool lflag;    // whether loss-cone distribution or not
      double xbeta;  // for loss-cone distribution
      x = *(t + i) * *(rm + i)*pow(yx*ylri,2)/(*(ran+i)* *(ran+i));
      debugg = x;
      if(*(loss0 + i) == 1 || *(loss0 + i) == 3)
      {
        lflag = true;
        xbeta = x* *(bfac + i);
      }
      else lflag = false;

      /* calculation of be*/
      *(be + i*NUM_BE) = gsl_sf_bessel_I0_scaled(x);
      *(be + i*NUM_BE + 1) = gsl_sf_bessel_I1_scaled(x);
      if(lflag)
      {
        *(be + (i+num)*NUM_BE) = gsl_sf_bessel_I0_scaled(xbeta);
        *(be + (i+num)*NUM_BE + 1) = gsl_sf_bessel_I1_scaled(xbeta);
      }
      *(maxb + i) = NUM_BE_MAX;
      *(maxb + (i+num)) = NUM_BE_MAX;
      for(int j = 2;j<NUM_BE_MAX;j++)
      {
        *(be + i*NUM_BE + j) = gsl_sf_bessel_In_scaled(j, x);
        if(pow(j,3)* *(be + i*NUM_BE + j) < unit)
        {
          *(maxb + i) = j;
          break;
        }
      }
      if(lflag)
      {
        for(int j = 2;j<NUM_BE_MAX;j++)
        {
          *(be + (i+num)*NUM_BE + j) = gsl_sf_bessel_In_scaled(j, xbeta);
          if(pow(j,3)* *(be + (i+num)*NUM_BE + j) < unit)
          {
            *(maxb + (i+num)) = j;
            break;
          }
        }
      }

      /* calculation of dbe and benn */
      *(dbe + i*NUM_BE) = *(be + i*NUM_BE + 1) - *(be + i*NUM_BE);
      for(int j = 1; j < *(maxb+i); j++) *(dbe + i*NUM_BE +j) = 0.5*( *(be + i*NUM_BE + j-1) + *(be + i*NUM_BE + j+1)) - *(be + i*NUM_BE + j);
      for(int j = 0;j<*(maxb+i);j++) *(benn + i*NUM_BE +j) = *(be + i*NUM_BE +j)/x*j*j;
      if(lflag)
      {
        *(dbe + (i+num)*NUM_BE) = *(be + (i+num)*NUM_BE + 1) - *(be + (i+num)*NUM_BE);
        for(int j = 1;j<*(maxb+i+num);j++) *(dbe + (i+num)*NUM_BE +j) = 0.5*( *(be + (i+num)*NUM_BE + j-1) + *(be + (i+num)*NUM_BE + j+1)) - *(be + (i+num)*NUM_BE + j);
        for(int j = 0;j<*(maxb+i+num);j++) *(benn + (i+num)*NUM_BE +j) = *(be + (i+num)*NUM_BE +j)/xbeta*j*j;
      }
    }
  }
  return 0;
}
