#include <complex.h>
#define __USE_BSD
#include <math.h>
/*
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
*/
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_sf_psi.h>
#include "../include/gtdispc.h"

#include <stdio.h>
#include <stdlib.h>

static int kk;

static complex small_zkm(double kappa, complex xi){
  complex xi2;
  complex A, B, C, add;
  int m;
  xi2 = xi*xi;
  A = I*sqrt(M_PI/kappa)*gsl_sf_gamma(kappa)/gsl_sf_gamma(kappa-0.5)*cpow(1+xi2/kappa,-kappa);
  B = -sqrt(M_PI)/kappa/gsl_sf_gamma(kappa-0.5)*xi;

  add = 1;
  m = 0;
  C = 0;
  while(cabs(add) > unit && m < 200){
    double aaa, sgn1, sgn2;
    gsl_sf_result rslt1, rslt2;
    if(kappa + m + 0.5 < 100){
      aaa = tgamma(kappa+m+0.5)/tgamma(m+1.5);
    }//    aaa = gsl_sf_gamma_e(kappa+m+0.5, &rslt1)/gsl_sf_gamma_e(m+1.5, &rslt2);
    else    aaa = exp(gsl_sf_lngamma_sgn_e(kappa+m+0.5, &rslt1, &sgn1) - gsl_sf_lngamma_sgn_e(m+1.5, &rslt2, &sgn2));
    add = pow(-1/kappa,m)*aaa*cpow(xi2,m);
    C += add;
    m++;
  }
  kk = m;

  return A+B*C;
}

static complex large_zkm(double kappa, complex xi){
  complex xi2;
  complex A, B, C, add;
  int m;
  xi2 = xi*xi;
  if(creal(xi) >= 0){
    A = sqrt(M_PI)*gsl_sf_gamma(kappa)/gsl_sf_gamma(kappa-0.5)*pow(kappa, kappa-0.5)*cpow(xi2+kappa,-kappa)*(I-tan(kappa*M_PI));
  }else{
    A = sqrt(M_PI)*gsl_sf_gamma(kappa)/gsl_sf_gamma(kappa-0.5)*pow(kappa, kappa-0.5)*cexp(-kappa*(clog(xi2+kappa)+2*M_PI*I))*(I-tan(kappa*M_PI));
  }
  B = -sqrt(M_PI)*gsl_sf_gamma(1.5-kappa)/xi;

  add = 1;
  m = 0;
  C = 0;
  while(cabs(add) > unit && m < 500){
    double aaa, sgn1, sgn2;
    gsl_sf_result rslt1, rslt2;
    if(1.5-kappa+m < 100){    aaa = gsl_sf_gamma(1.5-kappa+m)*gsl_sf_gamma(0.5-m);
      add = cpow(kappa/xi2, m)/aaa;
    }
    else{
      aaa = gsl_sf_lngamma_sgn_e(1.5-kappa+m, &rslt1, &sgn1) + gsl_sf_lngamma_sgn_e(0.5-m, &rslt2, &sgn2);
      //aaa = sgn1*sgn2*exp(m*log(kappa) - aaa);
      //C += aaa*cpow(xi2,-m);
      add = sgn1*sgn2/exp(aaa)*cpow(kappa/xi2,m);
    }
    C += add;
    m++;
  }

  kk = m;

  return A+B*C;
}

complex zkm(double kappa, complex xi){
  if(cabs(xi) <= sqrt(kappa)) return small_zkm(kappa, xi);
  else return large_zkm(kappa, xi);
}

/*
static complex pade2f1(complex a, complex b, complex c, complex z, int p, int q){
  gsl_vector_complex *A, *C, *y;
  gsl_matrix_complex *B;
  gsl_permutation *perm;
  gsl_complex gcmpl;

  A = gsl_vector_complex_alloc(p+q+1);
  gcmpl = gsl_complex_rect(1,0);;
  gsl_vector_complex_set(A, 0, gcmpl);
  for(int j1 = 1; j1 < p+q+1; j1++){
    gcmpl = gsl_complex_mul_real(gcmpl, (a+j1-1)*(b+j1-1)/(c+j1-1)/j1);
    gsl_vector_complex_set(A, j1, gcmpl);
  }

  C = gsl_vector_complex_alloc(p+q+1);
  B = gsl_matrix_complex_calloc(p+q+1, p+q+1);
  y = gsl_vector_complex_alloc(p+q+1);
  gsl_vector_complex_fprintf(stdout, A, "%e");

  gsl_vector_complex_memcpy(C,A);

  gcmpl = gsl_complex_rect(1,0);
  for(int j3 = 0; j3 < p+1; j3++) gsl_matrix_complex_set(B, j3, j3, gcmpl);
  for(int j5 = p+1; j5 < p+q+1; j5++)
    for(int j4 = j5 - p -1; j4 < p+q+1; j4++){
      gcmpl = gsl_vector_complex_get(A, p+1-j5+j4);
      gsl_matrix_complex_set(B , j4, j5, gsl_complex_negative(gcmpl));
    }

  perm = gsl_permutation_alloc(p+q+1);
  int i;
  gsl_linalg_complex_LU_decomp(B, perm, &i);
  gsl_linalg_complex_LU_solve(B, perm, C,y);
  
  gsl_vector_complex_free(A);
  gsl_matrix_complex_free(B);
  gsl_vector_complex_free(C);
  gsl_permutation_free(perm);
  
  gsl_vector_complex *pvec, *qvec, *numvec, *domvec;
  pvec = gsl_vector_complex_alloc(p+1);
  qvec = gsl_vector_complex_alloc(q);
  numvec = gsl_vector_complex_alloc(p+1);
  domvec = gsl_vector_complex_alloc(q);

  gcmpl = gsl_complex_rect(creal(z), cimag(z));
  for(int j6=0; j6 < p+1; j6++){
    gsl_vector_complex_set(pvec, j6, gsl_vector_complex_get(y, j6));
    gsl_vector_complex_set(numvec, j6, gsl_complex_pow_real(gcmpl, j6));
  }

  for(int j7=0; j7 < q; j7++){
    gsl_vector_complex_set(qvec, j7, gsl_vector_complex_get(y, j7+p));
    gsl_vector_complex_set(domvec, j7, gsl_complex_pow_real(gcmpl, j7));
  }

  complex ret;
  gsl_complex temp;
  gsl_blas_zdotu(qvec, domvec, &temp);
  temp = gsl_complex_add_real(temp, 1);
  gsl_blas_zdotu(pvec, numvec, &gcmpl);
  gcmpl = gsl_complex_div(gcmpl, temp);
  ret = GSL_REAL(gcmpl) + I*GSL_IMAG(gcmpl);
  gsl_vector_complex_free(pvec);
  gsl_vector_complex_free(qvec);
  gsl_vector_complex_free(numvec);
  gsl_vector_complex_free(domvec);

  return ret;
}
*/

double gamma_ratio(double a,double b){
  if(a < 100 && b < 100) return(tgamma(a))/tgamma(b);
  else{
    return exp(gsl_sf_lngamma(a) - gsl_sf_lngamma(b));
  }
}

static complex Atable[201];
static complex *Btable[201];
static int done = 0;

void create_kappa_table(){
  if(done != 0) return;
  done = 1;
  for(int i = 1; i <= 200; i++) Atable[i] = -(i -0.5)/2/pow(i, 1.5)*gamma_ratio(i+1, 2*i+1);
  for(int i = 1; i <= 200; i++){
    Btable[i] = (complex *) malloc(sizeof(complex)*(i+1));
    for(int m = 0; m <= i; m++)  *(Btable[i]+m) = gamma_ratio(i+m+1, m+1)*cpow(I,i-m);
  }
  
}

complex zk_int(int kappa, complex z){
  double A;
  complex arg, B;
  /*
  A = -(kappa -0.5)/2/pow(kappa, 1.5)*gamma_ratio(kappa+1, 2*kappa+1);
  arg = z/sqrt(kappa);
  B = 0;
  for(int m = 0; m <= kappa; m++)  B += gamma_ratio(kappa+m+1, m+1)*cpow(I,kappa-m)*cpow(2/(arg+I), kappa+1-m);
  */
  A = Atable[kappa];
  arg = z/sqrt(kappa);
  B = 0;
  for(int m = 0; m <= kappa; m++)  B += *(Btable[kappa]+m) *cpow(2/(arg+I), kappa+1-m);

  return A*B;
}

complex zkm_int(int kappa, complex z){
  return pow(kappa-1, 1.5)/(kappa-1.5)/sqrt(kappa)*zk_int(kappa-1, sqrt((kappa-1)/((double) kappa))*z);
}

complex zkmd_int(int kappa, complex z){
  return -2*((kappa-0.5)/kappa)*(1+z*sqrt((kappa+1)/((double) kappa))*zkm_int(kappa+1, sqrt((kappa+1)/((double) kappa))*z));
}

complex zkm_intp(double kappa, complex z){
  int kappa_f;
  double dif,ddif;
  complex zkm_l, zkm_h, zk_l, zk_h;
  complex dzkm_l, dzkm_h;
  double u0,u1,v0,v1;
  
  kappa_f = floor(kappa);
  zkm_l = zkm_int(kappa_f, z);
  zk_l  = zk_int(kappa_f, z);
  zkm_h = zkm_int(kappa_f+1, z);
  zk_h  = zk_int(kappa_f+1, z);
  
  dif = kappa - kappa_f;
  ddif = 1 - dif;
  // linear interpolation
  return zkm_l*(1-dif) + zkm_h*dif;


  // hermite interpolation
  /*
  double kappa_l, kappa_h;
  kappa_l = kappa_f;
  kappa_h = kappa_l + 1.;
  dzkm_l = zkm_l*(1 - 1.5/kappa_l + gsl_sf_psi(kappa_l+1) - gsl_sf_psi(kappa_l-0.5)) - zk_l;
  dzkm_h = zkm_h*(1 - 1.5/kappa_h + gsl_sf_psi(kappa_h+1) - gsl_sf_psi(kappa_h-0.5)) - zk_h;
  
  u0 = (1+2*dif)*ddif*ddif;
  u1 = (1-2*ddif)*dif*dif;
  v0 = -dif*ddif;
  v1 = -v0;
  
  return zkm_l*u0 + zkm_h*u1 + dzkm_l*v0 + dzkm_h*v1;
  */
}


/*
int main(){
  double x, y, error;
  complex zz, s0, s;
  calculate_unit();
  for(int i = 0; i <= 1000;i++){
    y = 0.01*(i- 500);
    for(int j = 0; j <= 1000; j++){
      x = 0.01*(j-500);
      zz = x + I*y;
      s0 = zkm(2.9, zz);
      s = zkm_intp(2.9, zz);
      error = cabs((s-s0)/s0)*100;
      printf("%e\t", error);
      //printf("%e\t", cimag(s0));
      //printf("%e\t%e\t%e\t%d\n", x, creal(s), cimag(s), kk);
      //s = zkm_int(4, x);
      //printf("%e\t%e\t%e\t%d\n", x, creal(s), cimag(s));
    }
    printf("\n");
  }
}
*/

