#include <complex.h>
#define __USE_BSD
#include <math.h>
#include "header.h"

/*
 * c_vec e_ratio(double theta, complex h1, complex h2, complex h3, complex h4, complex h5, complex h6){
 * input: theta(deg), h components
 * output: E vector, which is not normailized
 */


c_vec e_ratio(double theta, complex h1, complex h2, complex h3, complex h4, complex h5, complex h6){
  complex DDD[5][5];
  int nflag[4], iflag, jflag;
//-- CALCURATION OF MATRIX COMPONENT ------------------------------------
  double si,c,s2, sc, c2,t;
  t = theta/180*M_PI;
  si = sin(t);
  c = cos(t);
  s2 = si*si;
  sc = si*c;
  c2 = c*c;
  for(int i = 0; i<5; i++)
    for(int j = 0; j<5; j++) DDD[i][j] = 0;
  DDD[1][1] = h1*s2 - 2*h3*sc + h6*c2;
  DDD[1][2] = I*(h2*si - h5*c);
  DDD[1][3] = h1*sc + h3*(s2 - c2) - h6*sc;
  DDD[2][1] = -DDD[1][2];
  DDD[2][2] = h4;
  DDD[2][3] = -I*(h2*c + h5*si);
  DDD[3][1] = DDD[1][3];
  DDD[3][2] = -DDD[2][3];
  DDD[3][3] = h1*c2 + 2*h3*sc + h6*s2;
//------ MATRIX CONVERSION --------------------------------------------------
  for(int i = 1; i <= 3;i++) nflag[i] = i;
  for(int npivot = 1; npivot<=2; npivot++){
    double ddmax;
    iflag = npivot;
    jflag = npivot;
    // find the maximum element
    ddmax = cabs(DDD[npivot][npivot]);
    for(int i = npivot; i <= 3; i++){
      for(int j = npivot; j <= 3; j++){
        if(cabs(DDD[i][j]) > ddmax){
          ddmax = cabs(DDD[i][j]);
          iflag = i;
          jflag = j;
        }
      }
    }
    // exchange rows or columns
    for(int  j=1; j <= 3; j++){
      DDD[4][j]      = DDD[iflag][j];
      DDD[iflag][j]  = DDD[npivot][j];
      DDD[npivot][j] = DDD[4][j];
     }
    for(int i = 1; i <= 3; i++){
      DDD[i][4]      = DDD[i][jflag];
      DDD[i][jflag]  = DDD[i][npivot];
      DDD[i][npivot] = DDD[i][4];
    }
     // normalize the npibot-th row
    for(int j = 3; j >= npivot; j--) DDD[npivot][j] = DDD[npivot][j]/DDD[npivot][npivot];
    // sweep out the npibot-th column
    for(int i = npivot+1; i <= 3; i++){
      complex pbuff;
      pbuff = DDD[i][npivot];
      for(int j = npivot; j <= 3; j++) DDD[i][j] = DDD[i][j] - DDD[npivot][j]*pbuff;
    }
    // reorder E
    int ibuff;
    ibuff = nflag[jflag];
    nflag[jflag] = nflag[npivot];
    nflag[npivot] = ibuff;
  }
//---- RELATION BETWEEN EACH COMPONENT OF E(B) VECTOR--------------------
  complex pol[4];
  pol[nflag[1]] = -DDD[1][3];
  pol[nflag[2]] = -DDD[2][3];
  pol[nflag[3]] = 1;

  c_vec ret;
  ret.x = pol[1];
  ret.y = pol[2];
  ret.z = pol[3];
  return ret;
}
