#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <complex.h>

#define MAX_LINES 100
#define MAX_DATA 200

/* inputs
 * filename - file name
 * k - wave-number
 * w - angular frequency
 * r - growth rate
 */
/* return
 * the number of lines
 * -1 will be returned if an error is happened such as the file does not exist
 * 101 will be retured the number of lines is too many
 */
int load_kwr(char *filename, double ***k, complex ***w, complex ***h1, complex ***h2, complex ***h3, complex ***h4, complex ***h5, complex ***h6){
  FILE *fp;
  int nlines = 0;

  *k = (double **) malloc(sizeof(double *)*MAX_LINES);
  *w = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h1 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h2 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h3 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h4 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h5 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  *h6 = (complex **) malloc(sizeof(complex *)*MAX_LINES);
  if(!k || !w ) return -1;

  fp = fopen(filename, "r");
  if(fp == NULL) return -1;
  else{
    char text[1000];
    int i = 0;
    **k = (double *) malloc(sizeof(double)*MAX_DATA);
    **w = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h1 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h2 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h3 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h4 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h5 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    **h6 = (complex *) malloc(sizeof(complex)*MAX_DATA);
    while(fgets(text, 1000, fp)){
      if(*text == '\n'){
        if(i != MAX_DATA){
          *(*(*k + nlines) + i) = -DBL_MAX;
          *(*(*w + nlines) + i) = -DBL_MAX;
          *(*(*h1 + nlines) + i) = -DBL_MAX;
          *(*(*h2 + nlines) + i) = -DBL_MAX;
          *(*(*h3 + nlines) + i) = -DBL_MAX;
          *(*(*h4 + nlines) + i) = -DBL_MAX;
          *(*(*h5 + nlines) + i) = -DBL_MAX;
          *(*(*h6 + nlines) + i) = -DBL_MAX;
        }
        i = 0;
        nlines++;
        if(nlines < MAX_LINES){
          *(*k+nlines) = (double *) malloc(sizeof(double)*MAX_DATA);
          *(*w+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h1+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h2+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h3+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h4+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h5+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
          *(*h6+nlines) = (complex *) malloc(sizeof(complex)*MAX_DATA);
        }
      }else{
        double kk, wwr, wwi, hh1r, hh1i, hh2r, hh2i, hh3r, hh3i, hh4r, hh4i, hh5r, hh5i, hh6r, hh6i;
        sscanf(text, "%le %le %le %le %le %le %le %le %le %le %le %le %le %le %le\n", &kk, &wwr, &wwi, &hh1r, &hh1i, &hh2r, &hh2i, &hh3r, &hh3i, &hh4r, &hh4i, &hh5r, &hh5i, &hh6r, &hh6i); 
        *(*(*k + nlines) + i) = kk;
        *(*(*w + nlines) + i) = wwr;
        *(*(*w + nlines) + i) += I*wwi;
        *(*(*h1 + nlines) + i) = hh1r;
        *(*(*h1 + nlines) + i) += I*hh1i;
        *(*(*h2 + nlines) + i) = hh2r;
        *(*(*h2 + nlines) + i) += I*hh2i;
        *(*(*h3 + nlines) + i) = hh3r;
        *(*(*h3 + nlines) + i) += I*hh3i;
        *(*(*h4 + nlines) + i) = hh4r;
        *(*(*h4 + nlines) + i) += I*hh4i;
        *(*(*h5 + nlines) + i) = hh5r;
        *(*(*h5 + nlines) + i) += I*hh5i;
        *(*(*h6 + nlines) + i) = hh6r;
        *(*(*h6 + nlines) + i) += I*hh6i;
        i++;
      }
    }
  }

  return nlines;
}

void free_kwr(int nlines, double **k, complex **w, complex **h1, complex **h2, complex **h3, complex **h4, complex **h5, complex **h6){
  for(int i = 0; i < nlines; i++){
    free(*(k + i));
    free(*(w + i));
    free(*(h1 + i));
    free(*(h2 + i));
    free(*(h3 + i));
    free(*(h4 + i));
    free(*(h5 + i));
    free(*(h6 + i));
  }
  free(k);
  free(w);
  free(h1);
  free(h2);
  free(h3);
  free(h4);
  free(h5);
  free(h6);
}
