#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <complex.h>

/*
 * open_files_h
 * input: the prefix of each file(chars except _h?.txt)
 * return: void
 * You will be ready to read h components files and wave(including growth rate) files.
 *
 * close_files_h
 * input: void
 * return: void
 * You should terminate a program after using this.
 *
 * double read_h(double **kkk, complex **ww, complex **hh1, complex **hh2, complex **hh3, complex **hh4, complex **hh5, complex **hh6){
 * input: you should pass arguments where the data is stored
 * return: propagation angle
 */

static FILE *fpw, *fpr, *fpk, *fph1, *fph2, *fph3, *fph4, *fph5, *fph6;
static double propagation_angle;
static double theta_min, theta_max;
int Data_Num;


void open_files_h(const char *prefix){
  int underscore_start;
  char *filename;
  filename = (char *) malloc(sizeof(char)*(strlen(prefix)+7+1));
  strcpy(filename, prefix);

  underscore_start = strlen(prefix);
  strcpy(filename+underscore_start,"_h1.txt");
  fph1 = fopen(filename,"r");
  *(filename+underscore_start+2) = '2';
  fph2 = fopen(filename,"r");
  *(filename+underscore_start+2) = '3';
  fph3 = fopen(filename,"r");
  *(filename+underscore_start+2) = '4';
  fph4 = fopen(filename,"r");
  *(filename+underscore_start+2) = '5';
  fph5 = fopen(filename,"r");
  *(filename+underscore_start+2) = '6';
  fph6 = fopen(filename,"r");

  strcpy(filename+underscore_start,"_w.txt");
  fpw = fopen(filename,"r");
  *(filename+underscore_start+1) = 'r';
  fpr = fopen(filename,"r");
  *(filename+underscore_start+1) = 'k';
  fpk = fopen(filename,"r");

  FILE *fpd;
  *(filename+underscore_start+1) = 'd';
  fpd = fopen(filename,"r");
  fscanf(fpd,"%lf %lf %d", &theta_min, &theta_max, &Data_Num);
  fclose(fpd);

  free(filename);
  propagation_angle = theta_min - (theta_max - theta_min)/(Data_Num - 1);
}

void close_files_h(void){
  fclose(fph1);
  fclose(fph2);
  fclose(fph3);
  fclose(fph4);
  fclose(fph5);
  fclose(fph6);
  fclose(fpw);
  fclose(fpr);
  fclose(fpk);
}

static complex *w = NULL;
static double *kk = NULL;
static complex *h1 = NULL;
static complex *h2 = NULL;
static complex *h3 = NULL;
static complex *h4 = NULL;
static complex *h5 = NULL;
static complex *h6 = NULL;

double read_h(double **kkk, complex **ww, complex **hh1, complex **hh2, complex **hh3, complex **hh4, complex **hh5, complex **hh6){
  double real, imag;
  if(h1 == NULL){
    w = (complex *) malloc(sizeof(complex)*600);
    kk = (double *) malloc(sizeof(double)*600);
    h1 = (complex *) malloc(sizeof(complex)*600);
    h2 = (complex *) malloc(sizeof(complex)*600);
    h3 = (complex *) malloc(sizeof(complex)*600);
    h4 = (complex *) malloc(sizeof(complex)*600);
    h5 = (complex *) malloc(sizeof(complex)*600);
    h6 = (complex *) malloc(sizeof(complex)*600);
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph1, "%lf\t%lf\t", &real, &imag);
    *(h1 + i) = real + I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph2, "%lf\t%lf\t", &real, &imag);
    *(h2 + i) = real + I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph3, "%lf\t%lf\t", &real, &imag);
    *(h3 + i) = real + I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph4, "%lf\t%lf\t", &real, &imag);
    *(h4 + i) = real + I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph5, "%lf\t%lf\t", &real, &imag);
    *(h5 + i) = real + I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fph6, "%lf\t%lf\t", &real, &imag);
    *(h6 + i) = real + I*imag;
  }

  for(int i = 0; i < 600; i++){
    fscanf(fpw, "%lf\t", &real);
    *(w + i) = real;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fpr, "%lf\t", &imag);
    *(w + i) += I*imag;
  }
  for(int i = 0; i < 600; i++){
    fscanf(fpk, "%lf\t", &real);
    *(kk + i) = real;
  }

  propagation_angle += (theta_max - theta_min)/(Data_Num - 1);
  *hh1 = h1;
  *hh2 = h2;
  *hh3 = h3;
  *hh4 = h4;
  *hh5 = h5;
  *hh6 = h6;
  *ww = w;
  *kkk = kk;

  return propagation_angle;
}













