#include <gtk/gtk.h>
#define __USE_BSD
#include <math.h>
#include <stdbool.h>
#include "graph.h"

typedef struct{
  double xmin, xmax;
  double ymin, ymax;
  double cbmin, cbmax;
  int length;
  double *x1;
  double *y1;
  double *x2;
  double *y2;
  double *val;
  gchar *xlabel, *ylabel;
} contour_data;

static gboolean cb_expose_event(GtkWidget *canvas, GdkEventExpose *event, contour_data *data){
  GdkWindow *drawable = canvas->window;
  cairo_t *cr;
  const double axis_width  = 3;
  double xmin, xmax, ymin, ymax, cbmin, cbmax;
  double xoffset, yoffset, xscale, yscale;
  double uwidth, uheight;
  gint width, height;

  cr = gdk_cairo_create(drawable);
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);
  xmin = data->xmin;
  xmax = data->xmax;
  ymin = data->ymin;
  ymax = data->ymax;
  cbmin = data->cbmin;
  cbmax = data->cbmax;
  xoffset = width*0.12;
  yoffset = height*0.88;
  xscale = width*0.8/(xmax - xmin);
  yscale = height*0.8/(ymax - ymin);

  // draw background
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_rectangle(cr, width*0.01, height*0.01, width*0.98, height*0.98);
  cairo_fill(cr);

  //draw axes
  cairo_set_line_width(cr, axis_width);
  cairo_set_source_rgb(cr, 0, 1, 0);
  cairo_move_to(cr, xoffset, yoffset);
  cairo_line_to(cr, xoffset + xscale*(xmax-xmin), yoffset);
  cairo_move_to(cr, xoffset, yoffset);
  cairo_line_to(cr, xoffset, yoffset - yscale*(ymax - ymin));
  cairo_stroke(cr);

  cairo_rectangle(cr, xoffset, yoffset - yscale*(ymax - ymin), xscale*(xmax - xmin), yscale*(ymax - ymin));
  cairo_clip(cr);


  for(int i = 0; i < data->length; i++){
    double color;
    double x1,y1,x2,y2;
    color = (*(data->val + i) - cbmin)/(cbmax - cbmin);
    cairo_set_source_rgb(cr, color, 0, 0);
    x1 = xoffset + (*(data->x1 +i) - xmin)*xscale;
    y1 = yoffset - (*(data->y1 +i) - ymin)*yscale;
    x2 = xoffset + (*(data->x2 +i) - xmin)*xscale;
    y2 = yoffset - (*(data->y2 +i) - ymin)*yscale;
    cairo_rectangle(cr, x1, y2, x2 - x1, y1 - y2);
    cairo_stroke_preserve(cr);
    cairo_fill(cr);
  }

  cairo_reset_clip(cr);
  // draw subaxes
  {
    double powerx,powery, xsubwidth, ysubheight;
    double temp;
    double xdash[] = {10,10}, ydash[] = {10,10};
    double fontsize;
    cairo_set_dash(cr, xdash, 2,0);
    fontsize = height*0.05;
    fontsize = fontsize > 2 ? fontsize : 2;
    cairo_set_font_size(cr,fontsize);
    temp = log10((xmax - xmin)/5);
    powerx = floor(temp);
    xsubwidth = pow(10,temp - powerx);
    if(xsubwidth <= 1.6) xsubwidth = 1;
    else if(xsubwidth <= 2.4) xsubwidth = 2;
    else if(xsubwidth <= 4) xsubwidth = 2.5;
    else if (xsubwidth <= 8) xsubwidth = 5;
    else xsubwidth = 10;
    xsubwidth = xsubwidth*pow(10,powerx);
    if(fontsize/2*8 > xsubwidth*xscale) fontsize = xsubwidth*xscale/8*2*0.8;
    cairo_set_font_size(cr,fontsize);
    temp = ceil(xmin/xsubwidth)*xsubwidth;
    while(temp <= xmax){
      char text[21];
      cairo_set_source_rgba(cr, 0, 1, 0, 0.3);
      cairo_move_to(cr, xoffset+xscale*(temp - xmin), yoffset);
      cairo_line_to(cr, xoffset+xscale*(temp - xmin), yoffset-yscale*(ymax - ymin));
      cairo_stroke(cr);
      cairo_set_source_rgba(cr, 0, 1, 0, 1);
      cairo_move_to(cr, xoffset+xscale*(temp - xmin) - fontsize*2, yoffset + fontsize);
      snprintf(text,20,"%.2e", temp);
      cairo_show_text(cr, text);
      cairo_stroke(cr);
      temp += xsubwidth;
    }
    fontsize = width*0.02;
    fontsize = fontsize > 2 ? fontsize : 2;
    cairo_set_font_size(cr,fontsize);
    cairo_set_dash(cr, xdash, 2,0);
    temp = log10((ymax - ymin)/5);
    powery = floor(temp);
    ysubheight = pow(10,temp - powery);
    if(ysubheight <= 1.6) ysubheight = 1;
    else if(ysubheight <= 2.4) ysubheight = 2;
    else if(ysubheight <= 4) ysubheight = 2.5;
    else if (ysubheight <= 8) ysubheight = 5;
    else ysubheight = 10;
    ysubheight = ysubheight*pow(10,powery);
    temp = ceil(data->ymin/ysubheight)*ysubheight;
    while(temp <= data->ymax){
      char text[21];
      cairo_move_to(cr, xoffset, yoffset-(temp - data->ymin)*yscale);
      cairo_set_source_rgba(cr, 0, 1, 0, 0.3);
      cairo_line_to(cr, xoffset+xscale*(xmax - xmin), yoffset-(temp- data->ymin)*yscale);
      cairo_stroke(cr);
      cairo_set_source_rgba(cr, 0, 1, 0, 1);
      cairo_move_to(cr, xoffset - 4*fontsize, yoffset-(temp - data->ymin)*yscale);
      snprintf(text,20,"%.2e", temp);
      cairo_show_text(cr, text);
      cairo_stroke(cr);
      temp += ysubheight;
    }
    cairo_stroke(cr);
    cairo_set_dash(cr, NULL, 0, 0);
  }

  // draw labels
  if(data->xlabel != NULL){
    cairo_move_to(cr, width/2, height*0.96);
    cairo_show_text(cr,data->xlabel);
  }
  if(data->ylabel != NULL){
    cairo_save(cr);
    cairo_move_to(cr, width*0.03, height/2);
    cairo_rotate(cr, -M_PI/2);
    cairo_show_text(cr,data->ylabel);
    cairo_restore(cr);
  }
}

void set_c_range(GtkWidget *canvas, double minimum, double maximum, int range){
  contour_data *data = (contour_data*) g_object_get_data(G_OBJECT(canvas), "contour_data");
  if (data == NULL) return;
  switch (range){
    case 0:
      data->xmin = minimum;
      data->xmax = maximum;
      break;
    case 1:
      data->ymin = minimum;
      data->ymax = maximum;
      break;
    case 2:
      data->cbmin = minimum;
      data->cbmax = maximum;
      break;
  }

}

void set_c_values(GtkWidget *canvas, double *x1, double *y1, double *x2, double *y2, double *val, int length){
  contour_data *data = (contour_data*) g_object_get_data(G_OBJECT(canvas), "contour_data");
  if (data == NULL) return;
  data->x1 = x1;
  data->y1 = y1;
  data->x2 = x2;
  data->y2 = y2;
  data->val = val;
  data->length = length;
}

void set_c_label(GtkWidget *canvas, gchar *label, gint axis){
  contour_data *data = (contour_data*) g_object_get_data(G_OBJECT(canvas), "contour_data");
  if (data == NULL) return;
  if(axis == 0){
    g_free(data->xlabel);
    data->xlabel = g_strdup(label);
  }else if(axis == 1){
    g_free(data->ylabel);
    data->ylabel = g_strdup(label);
  }
}


GtkWidget *my_contour(void){
  GtkWidget *canvas;
  contour_data *data;

  canvas = gtk_drawing_area_new();
  data = g_new0(contour_data, 1);
  data->xmin = 0;
  data->xmax = 1;
  data->ymin = 0;
  data->ymax = 1;
  data->cbmin = 0;
  data->cbmax = 1;
  data->x1 = NULL;
  data->y1 = NULL;
  data->x2 = NULL;
  data->y2 = NULL;
  data->val = NULL;
  data->length = 0;
  data->xlabel = NULL;
  data->ylabel = NULL;

  g_object_set_data(G_OBJECT(canvas), "contour_data", data);
  g_signal_connect(G_OBJECT(canvas), "expose-event", G_CALLBACK(cb_expose_event), data);
  return canvas;
}
