#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <complex.h>

#include <gtdispc.h>

/*
 * void_for_all_direction(char* filename)
 * input: prefix of the file where the result will be stored
 * output: void
 *
 * This program obtains the maximum growth rate and corresponding values such as h components and frequencies and k.
 * If kupdap fails in getting the dispersion relation at certain k, this program substitutes -DBL_MAX.
 * Calculation is done from min k(xcs) to max k(xcl) and from 0 deg to 90 deg.
 * Results are stored in files.
 */
void for_all_direction(char* filename, double t_min, double t_max, int t_n){
  const int division = 600;
  double min_k, max_k;
  FILE *fp;
  char text[1024];
  int nline;
  double wavenum, freq, growth;
  complex h1, h2, h3, h4, h5, h6;
  double pre_freq, pre_growth;
  complex pre_h1, pre_h2, pre_h3, pre_h4, pre_h5, pre_h6;
  long int n_wavenum, pre_nwavenum;
  double *c_freq, *c_growth;
  complex *c_h1, *c_h2, *c_h3, *c_h4, *c_h5, *c_h6;
  double deltak;
  char filename2[1024];

  // process inputs
  min_k = xcs;
  max_k = xcl;

  snprintf(filename2, 1023, "%s.dat", filename);
  fp = fopen(filename2, "w");
  if(fp == NULL){
    printf("The file %s cannot be opened\n", filename);
    return;
  }

  deltak = (max_k - min_k)/(division - 1);

  c_freq    = (double *) malloc(sizeof(double) * division);
  c_growth    = (double *) malloc(sizeof(double) * division);
  c_h1    = (complex *) malloc(sizeof(complex) * division);
  c_h2    = (complex *) malloc(sizeof(complex) * division);
  c_h3    = (complex *) malloc(sizeof(complex) * division);
  c_h4    = (complex *) malloc(sizeof(complex) * division);
  c_h5    = (complex *) malloc(sizeof(complex) * division);
  c_h6    = (complex *) malloc(sizeof(complex) * division);

  // start reading
  for(int itheta = 0; itheta < t_n; itheta += 1){
    theta = (t_max - t_min)/(t_n-1) * itheta + t_min;
    calculate_unit();
    get_basic_parameters();
    ans = wkinit_v(theta);
    ans = wkcurv_v(ans, theta);

    for(int i = 0; i < division; i++) *(c_freq + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_growth + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h1 + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h2 + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h3 + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h4 + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h5 + i) = -DBL_MAX;
    for(int i = 0; i < division; i++) *(c_h6 + i) = -DBL_MAX;

    for(int iline = 0; iline < ans; iline++){
      pre_nwavenum = -1;
      for(int j = 0; j < 200; j++){
        if(*(rrr[iline] + j) != -DBL_MAX);
        else {
          classify(rrr[iline], fff[iline], ddd1[iline], ddd2[iline], ddd3[iline], ddd4[iline], ddd5[iline], ddd6[iline], polarity[iline], eee_x[iline], eee_y[iline], eee_z[iline], j);
          break;
        }
        if(j == 199){
          classify(rrr[iline], fff[iline], ddd1[iline], ddd2[iline], ddd3[iline], ddd4[iline], ddd5[iline], ddd6[iline], polarity[iline], eee_x[iline], eee_y[iline], eee_z[iline], 200);
        }
      }
      for(int j = 0; j < 200; j++){
        wavenum = *(rrr[iline] + j);
        if(wavenum == -DBL_MAX) break;
        else{
          complex fct1, fct2;
          freq = creal(*(fff[iline] + j));
          growth = cimag(*(fff[iline] + j));
          fct1 = fabs(wc)/(freq + I*growth);
          fct2 = fct1*fct1;
          h1 = *(ddd1[iline] + j);
          h2 = *(ddd2[iline] + j)*fct1;
          h3 = *(ddd3[iline] + j)*fct1;
          h4 = *(ddd4[iline] + j)*fct2;
          h5 = *(ddd5[iline] + j)*fct2;
          h6 = *(ddd6[iline] + j)*fct2;

          n_wavenum = lround((wavenum - min_k)/deltak);
          if(n_wavenum < 0 || n_wavenum > division -1) continue;
          if(pre_nwavenum < 0){
            ; // do nothing here
          }else{
            for(int i = pre_nwavenum + 1; i <= n_wavenum; i++){
              double p_freq, p_growth;
              complex p_h1, p_h2, p_h3, p_h4, p_h5, p_h6;
              p_freq = (freq - pre_freq)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_freq;
              p_growth = (growth - pre_growth)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_growth;
              p_h1 = (h1 - pre_h1)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h1;
              p_h2 = (h2 - pre_h2)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h2;
              p_h3 = (h3 - pre_h3)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h3;
              p_h4 = (h4 - pre_h4)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h4;
              p_h5 = (h5 - pre_h5)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h5;
              p_h6 = (h6 - pre_h6)/(n_wavenum - pre_nwavenum)*(i - pre_nwavenum) + pre_h6;
              if(*(c_growth + i) < p_growth && p_freq > -0.0001){//&& p_freq < 0.0625){
                *(c_growth + i) = p_growth;
                *(c_freq + i) = p_freq;
                *(c_h1 + i) = p_h1;
                *(c_h2 + i) = p_h2;
                *(c_h3 + i) = p_h3;
                *(c_h4 + i) = p_h4;
                *(c_h5 + i) = p_h5;
                *(c_h6 + i) = p_h6;
              }
            }
          }
          pre_nwavenum = n_wavenum;
          pre_freq = freq;
          pre_growth = growth;
          pre_h1 = h1;
          pre_h2 = h2;
          pre_h3 = h3;
          pre_h4 = h4;
          pre_h5 = h5;
          pre_h6 = h6;
        }
      }
    }
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", (xcl-xcs)/division*i + xcs);
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", *(c_freq + i));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", *(c_growth + i));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h1+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h1+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h2+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h2+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h3+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h3+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h4+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h4+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h5+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h5+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", creal(*(c_h6+i)));
    fprintf(fp, "\n");
    for(int i = 0; i < division; i++) fprintf(fp, "%e\t", cimag(*(c_h6+i)));
    fprintf(fp, "\n");
    if(itheta%10 == 0) printf("%d\n", itheta);
  }
  fclose(fp);
  free(c_growth);
  free(c_freq);
  free(c_h1);
  free(c_h2);
  free(c_h3);
  free(c_h4);
  free(c_h5);
  free(c_h6);
}

