#include <gtk/gtk.h>
#include <math.h>
#include <stdlib.h>
#include <complex.h>
#include "graph.h"
#include "state.h"
#include <gtdispc.h>
#include <glib.h>
#include <glib/gstdio.h>

/*
  memo
  When you change the species of particle, your modification is not saved.
*/

static gulong id_changed;

static GtkWidget *B_cv, *B_wc, *B_theta;
static GtkWidget *B_q, *B_qm, *B_drift, *B_wpd, *B_para, *B_perp, *B_ppp, *B_bfac, *B_kappa, *C_mode;
static GtkWidget *B_frmin, *B_frmax, *B_nfr, *B_fimin, *B_fimax, *B_nfi, *B_kmin, *B_kmax, *B_nk;
static GtkWidget *B_name, *Combo;

static int istate = 0;

option_set OS;

static void cb_changed(GtkWidget *combo, gpointer user_data);
static void override_val(void){
  const gchar *text;
  gchar text2[25];

  text = gtk_entry_get_text(GTK_ENTRY(B_q));
  *(an + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_qm));
  *(qm + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_wpd));
  *(wpd + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_para));
  *(path + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_perp));
  *(peth + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_drift));
  *(spa + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_ppp));
  *(ppp + istate) = strtod(text, NULL);
  //text = gtk_entry_get_text(GTK_ENTRY(B_loss0));
  //*(loss0 + istate) = strtod(text, NULL);
  *(loss0 + istate) = gtk_combo_box_get_active(GTK_COMBO_BOX(C_mode));
  text = gtk_entry_get_text(GTK_ENTRY(B_kappa));
  *(kappa + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_bfac));
  *(bfac + istate) = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_name));
  snprintf(*(sname + istate), 24, "%s", text);
  g_signal_handler_disconnect(G_OBJECT(Combo), id_changed);
  gtk_combo_box_remove_text(GTK_COMBO_BOX(Combo), istate);
  snprintf(text2, 24, "%d:%s", istate+1, *(sname + istate));
  gtk_combo_box_insert_text(GTK_COMBO_BOX(Combo), istate, text2);
  id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);
}

static void update_val1(void){
  const gchar *text;

  text = gtk_entry_get_text(GTK_ENTRY(B_cv));
  cv = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_wc));
  wc = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_theta));
  theta = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_frmin));
  cxi = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_frmax));
  cxa = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_fimin));
  cyi = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_fimax));
  cya = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_kmin));
  xcs = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_kmax));
  xcl = strtod(text, NULL);
  text = gtk_entry_get_text(GTK_ENTRY(B_nfr));
  nx = atoi(text);
  text = gtk_entry_get_text(GTK_ENTRY(B_nfi));
  ny = atoi(text);
  text = gtk_entry_get_text(GTK_ENTRY(B_nk));
  nk = atoi(text);
}

static void destroy(GtkWidget *w, gpointer p){
  g_message("Program closed");
  gtk_main_quit();
}

static gint parse_data(gchar* filename){
  FILE *fp=NULL;
  fp = g_fopen(filename, "r");
  int num_before;
  num_before = num;
  if(fp){
    int result;
    result = fscanf(fp, "%d %lf %lf\n", &num, &wc, &cv);
    if(result != 3 || num <= 0 || wc == 0 || cv <= 0) goto file_is_dead;
    result = fscanf(fp, "%d %d %d\n", &nx, &ny, &nk);
    if(result != 3 || nx <= 0 || ny <= 0 || nk <= 0) goto file_is_dead;
    result = fscanf(fp, "%lf\n", &theta);
    if(result != 1 || theta > 90 || theta < 0) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &xcs, &xcl);
    if(result != 2 || xcs < 0 || xcl < xcs) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &cxi, &cxa);
    if(result != 2 ||  cxa < cxi) goto file_is_dead;
    result = fscanf(fp, "%lf %lf\n", &cyi, &cya);
    if(result != 2 ||  cya < cyi) goto file_is_dead;

    initialize_parameters(num);
    for(int i = 0; i < num_before; i++) free(*(sname + i));
    free(sname);
    sname = (char **) malloc(sizeof(char *) * num);
    //if(!an || !wpd || !qm || !peth || !path || !spa || !loss0 || !bfac || !ppp) goto malloc_error;
    for(int i = 0; i < num; i++){
      *(sname + i) = (char *) malloc(sizeof(char) * 21);
      fscanf(fp, "%20s\n", *(sname + i));
      result = fscanf(fp, "%lf %lf %lf\n", an+i, wpd+i, qm+i);
      if(result != 3) goto file_is_dead;
      result = fscanf(fp, "%lf %lf %lf\n", peth+i, path+i, spa+i);
      if(result != 3) goto file_is_dead;
      result = fscanf(fp, "%d %d %lf %lf\n", loss0+i,kappa+i,  bfac+i, ppp+i);
      if(result != 4) goto file_is_dead;
    }
    char text[25];
    snprintf(text,20,"%e", wc);
    gtk_entry_set_text(GTK_ENTRY(B_wc), text);
    snprintf(text,20,"%e", cv);
    gtk_entry_set_text(GTK_ENTRY(B_cv), text);
    snprintf(text,20,"%e", theta);
    gtk_entry_set_text(GTK_ENTRY(B_theta), text);
    snprintf(text,20,"%e", *an);
    gtk_entry_set_text(GTK_ENTRY(B_q), text);
    snprintf(text,20,"%e", *qm);
    gtk_entry_set_text(GTK_ENTRY(B_qm), text);
    snprintf(text,20,"%e", *wpd);
    gtk_entry_set_text(GTK_ENTRY(B_wpd), text);
    snprintf(text,20,"%e", *spa);
    gtk_entry_set_text(GTK_ENTRY(B_drift), text);
    snprintf(text,20,"%e", *path);
    gtk_entry_set_text(GTK_ENTRY(B_para), text);
    snprintf(text,20,"%e", *peth);
    gtk_entry_set_text(GTK_ENTRY(B_perp), text);
    //snprintf(text,20,"%d", *loss0);
    //gtk_entry_set_text(GTK_ENTRY(B_loss0), text);
    gtk_combo_box_set_active(GTK_COMBO_BOX(C_mode), *loss0);
    snprintf(text,20,"%d", *kappa);
    gtk_entry_set_text(GTK_ENTRY(B_kappa), text);
    snprintf(text,20,"%e", *bfac);
    gtk_entry_set_text(GTK_ENTRY(B_bfac), text);
    snprintf(text,20,"%e", *ppp);
    gtk_entry_set_text(GTK_ENTRY(B_ppp), text);
    snprintf(text,20,"%e", cxi);
    gtk_entry_set_text(GTK_ENTRY(B_frmin), text);
    snprintf(text,20,"%e", cxa);
    gtk_entry_set_text(GTK_ENTRY(B_frmax), text);
    snprintf(text,20,"%d", nx);
    gtk_entry_set_text(GTK_ENTRY(B_nfr), text);
    snprintf(text,20,"%e", cyi);
    gtk_entry_set_text(GTK_ENTRY(B_fimin), text);
    snprintf(text,20,"%e", cya);
    gtk_entry_set_text(GTK_ENTRY(B_fimax), text);
    snprintf(text,20,"%d", ny);
    gtk_entry_set_text(GTK_ENTRY(B_nfi), text);
    snprintf(text,20,"%e", xcs);
    gtk_entry_set_text(GTK_ENTRY(B_kmin), text);
    snprintf(text,20,"%e", xcl);
    gtk_entry_set_text(GTK_ENTRY(B_kmax), text);
    snprintf(text,20,"%d", nk);
    gtk_entry_set_text(GTK_ENTRY(B_nk), text);
    gtk_entry_set_text(GTK_ENTRY(B_name), *sname);

    g_signal_handler_disconnect(G_OBJECT(Combo), id_changed);
    for(int i = 0; i <= num_before; i++){
      gtk_combo_box_remove_text(GTK_COMBO_BOX(Combo), 0);
    }
    for(int i = 0; i < num; i++){
      gchar text[25];
      snprintf(text, 24, "%d:%s", i+1, *(sname + i));
      gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), text);
    }
    gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "Add");
    gtk_combo_box_set_active(GTK_COMBO_BOX(Combo), 0);
    id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);

    istate = 0;
  }else{
    return 1;
  }
  return 0;
  file_is_dead:
  // here should be implemented
    return 2;
}

static void cb_store(GtkWidget *w, GtkWindow *window){
  GtkWidget *dialog;
  gint response;
  update_val1();
  if(istate != -1) override_val();
  dialog = gtk_file_chooser_dialog_new("Restore parameters", GTK_WINDOW(window), GTK_FILE_CHOOSER_ACTION_SAVE,\
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
  gtk_widget_show_all(dialog);
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  if ( response == GTK_RESPONSE_ACCEPT){
    gchar *filename;
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    FILE *fp;
    fp = fopen(filename, "w");
    if(fp){
      fprintf(fp, "%d %e %e\n", num, wc, cv);
      fprintf(fp, "%d %d %d\n", nx, ny, nk);
      fprintf(fp, "%e\n", theta);
      fprintf(fp, "%e %e\n", xcs, xcl);
      fprintf(fp, "%e %e\n", cxi, cxa);
      fprintf(fp, "%e %e\n", cyi, cya);
      for(int i = 0;i < num; i++){
        fprintf(fp, "%s\n", *(sname + i));
        fprintf(fp, "%e %e %e\n", *(an + i), *(wpd + i), *(qm + i));
        fprintf(fp, "%e %e %e\n", *(peth + i), *(path + i), *(spa + i));
        fprintf(fp, "%d %d %e %e\n", *(loss0 + i), *(kappa+i), *(bfac + i), *(ppp + i));
      }
      gchar *basename, *title;
      basename = g_path_get_basename(filename);
      title = g_strdup_printf("KUPDAP: %s", basename);
      gtk_window_set_title(window, title);
      g_free(title);
      g_free(basename);
      fclose(fp);
    }else{
      g_message("saving a file is failed");
    }
    g_free(filename);
  }
  gtk_widget_destroy(dialog);
}

static void cb_open(GtkWidget *w, GtkWindow *window){
  GtkWidget *dialog;
  gint response;
  GtkFileFilter *filter, *nofilter;
  dialog = gtk_file_chooser_dialog_new("Restore parameters", GTK_WINDOW(window), GTK_FILE_CHOOSER_ACTION_OPEN,\
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  filter = gtk_file_filter_new();
  nofilter = gtk_file_filter_new();
  gtk_file_filter_set_name(filter, "Parameter Files");
  gtk_file_filter_set_name(nofilter, "ALL Files");
  gtk_file_filter_add_pattern(filter, "*.prm");
  gtk_file_filter_add_pattern(nofilter, "*");
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), nofilter);
  gtk_widget_show_all(dialog);
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  if ( response == GTK_RESPONSE_ACCEPT){
    int result;
    gchar *filename, *basename, *title;
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    result = parse_data(filename);
    basename = g_path_get_basename(filename);
    title = g_strdup_printf("KUPDAP: %s", basename);
    gtk_window_set_title(window, title);
    g_free(title);
    g_free(basename);
    g_free(filename);
    if(result == 1) g_message("fail to load");
    else if(result == 2) g_message("fail to parse");
  }
  gtk_widget_destroy(dialog);
}

reserved_state init_state;

static void copy_state(){
  init_state.cv = cv;
  init_state.theta = theta;
  init_state.wc = wc;
  init_state.val = val;
  init_state.b = b;
  init_state.num = num;

  if(init_state.an != NULL){
    free(init_state.an);
    free(init_state.loss0);
  }
  init_state.an = (double *) malloc(sizeof(double)*num*10);
  init_state.loss0 = (int *) malloc(sizeof(int)*num*2);
  init_state.wpd  = init_state.an+1*num;
  init_state.qm   = init_state.an+2*num;
  init_state.peth = init_state.an+3*num;
  init_state.path = init_state.an+4*num;
  init_state.spa  = init_state.an+5*num;
  init_state.bfac = init_state.an+6*num;
  init_state.ppp  = init_state.an+7*num;
  init_state.rn   = init_state.an+8*num;
  init_state.wp   = init_state.an+9*num;
  init_state.kappa   = init_state.loss0+1*num;
  for(int i = 0; i < num; i++){
    *(init_state.an+i)   = *(an+i);
    *(init_state.wpd+i)  = *(wpd+i);
    *(init_state.qm+i)   = *(qm+i);
    *(init_state.peth+i) = *(peth+i);
    *(init_state.path+i) = *(path+i);
    *(init_state.spa+i)  = *(spa+i);
    *(init_state.bfac+i) = *(bfac+i);
    *(init_state.ppp+i)  = *(ppp+i);
    *(init_state.rn+i)   = *(rn+i);
    *(init_state.wp+i)   = *(wp+i);
    *(init_state.loss0+i)= *(loss0+i);
    *(init_state.kappa+i)= *(kappa+i);
  }

  init_state.kmin = xcs;
  init_state.kmax = xcl;
  init_state.wmin = cxi;
  init_state.wmax = cxa;
  init_state.rmin = cyi;
  init_state.rmax = cya;
  
}


dcurve* DCURVE = NULL;

static void cb_main2(GtkWidget *w, gpointer user_data){
  if(DCURVE) g_free(DCURVE); // should be fixed
  ans = 0;
  if(MAIN2 == NULL) MAIN2 = main2_new();
  else main2_reset(MAIN2);
  clear_line(CanvasWK);
  clear_line(CanvasGK);
  clear_line(CanvasGW);
  normal_x = 1;
  normal_y = 1;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(POL_BUTTON), FALSE); // this would emit the "clicked" signal
  gtk_widget_show_all(MAIN2);

  update_val1();
  if(istate >= 0)override_val();
  calculate_unit();
  get_basic_parameters();
  ans = wkinit_v(theta);
  ans = wkcurv_v(ans, theta);

  copy_state();
  set_xrange(CanvasWK, xcs, xcl);
  set_yrange(CanvasWK, cxi, cxa);
  set_xrange(CanvasGK, xcs, xcl);
  set_yrange(CanvasGK, cyi, cya);
  set_xrange(CanvasGW, cxi, cxa);
  set_yrange(CanvasGW, cyi, cya);
  gchar text[21];
  snprintf(text, 20, "%e", cxi);
  gtk_entry_set_text(GTK_ENTRY(R_frmin), text);
  snprintf(text, 20, "%e", cxa);
  gtk_entry_set_text(GTK_ENTRY(R_frmax), text);
  snprintf(text, 20, "%e", cyi);
  gtk_entry_set_text(GTK_ENTRY(R_fimin), text);
  snprintf(text, 20, "%e", cya);
  gtk_entry_set_text(GTK_ENTRY(R_fimax), text);
  snprintf(text, 20, "%e", xcs);
  gtk_entry_set_text(GTK_ENTRY(R_kmin), text);
  snprintf(text, 20, "%e", xcl);
  gtk_entry_set_text(GTK_ENTRY(R_kmax), text);

  double *f, *g;
  int length;
  f = (double *)malloc(sizeof(double) * 200 * ans);
  g = (double *)malloc(sizeof(double) * 200 * ans);
  DCURVE = (dcurve *)g_malloc(sizeof(dcurve)*ans);
  int r = 0, gr = 0, b = 0;
  int d0, d1, d2, d3, d4, d5, d6, d7, d8;
  r = 1;
  d0 = 1;
  d1 = d0 << 1; 
  d2 = d0 << 2; 
  d3 = d0 << 3; 
  d4 = d0 << 4; 
  d5 = d0 << 5; 
  d6 = d0 << 6; 
  d7 = d0 << 7; 
  for(int i = 0; i < ans; i++){
    length = 200;
    double rr, gg, bb;
    rr = 0;
    gg = 0;
    bb = 0;
    for(int j = 0; j < 200; j++){
      if(*(rrr[i] + j) != -DBL_MAX){
        *(f+i*200+j) = creal(*(fff[i] + j));
        *(g+i*200+j) = cimag(*(fff[i] + j));
      }
      else {
        length = j;
        classify(rrr[i], fff[i], ddd1[i], ddd2[i], ddd3[i], ddd4[i], ddd5[i], ddd6[i], polarity[i], eee_x[i], eee_y[i], eee_z[i],length);
        break;
      }
      if(j == 199){
        classify(rrr[i], fff[i], ddd1[i], ddd2[i], ddd3[i], ddd4[i], ddd5[i], ddd6[i], polarity[i], eee_x[i], eee_y[i], eee_z[i], 200);
      }
    }
    rr += ((d0 & r) > 0) ? d7 :0;
    rr += ((d1 & r) > 0) ? d6 :0;
    rr += ((d2 & r) > 0) ? d5 :0;
    rr += ((d3 & r) > 0) ? d4 :0;
    rr += ((d4 & r) > 0) ? d3 :0;
    rr += ((d5 & r) > 0) ? d2 :0;
    rr += ((d6 & r) > 0) ? d1 :0;
    rr += ((d7 & r) > 0) ? d0 :0;
    rr /= 256*2;
    gg += ((d0 & gr) > 0) ? d7 :0;
    gg += ((d1 & gr) > 0) ? d6 :0;
    gg += ((d2 & gr) > 0) ? d5 :0;
    gg += ((d3 & gr) > 0) ? d4 :0;
    gg += ((d4 & gr) > 0) ? d3 :0;
    gg += ((d5 & gr) > 0) ? d2 :0;
    gg += ((d6 & gr) > 0) ? d1 :0;
    gg += ((d7 & gr) > 0) ? d0 :0;
    gg /= 256*2;
    bb += ((d0 & b) > 0) ? d7 :0;
    bb += ((d1 & b) > 0) ? d6 :0;
    bb += ((d2 & b) > 0) ? d5 :0;
    bb += ((d3 & b) > 0) ? d4 :0;
    bb += ((d4 & b) > 0) ? d3 :0;
    bb += ((d5 & b) > 0) ? d2 :0;
    bb += ((d6 & b) > 0) ? d1 :0;
    bb += ((d7 & b) > 0) ? d0 :0;
    bb /= 256*2;
    //rr = 0.3;
    //gg = 0.3;
    //bb = 0.3;
    switch (i%3){
      case 0:
        r++;
        rr += 0.5;
        break;
      case 1:
        gr++;
        gg += 0.5;
        break;
      case 2:
        b++;
        bb += 0.5;
        break;
    }
    // sort
    for(int j = 0; j < length-1; j++){
      int mm;
      double temp;
      mm = j;
      for(int jj = j+1; jj < length; jj++){
        mm = *(rrr[i] + mm) > *(rrr[i] + jj) ? jj : mm;
      }
      temp = *(rrr[i] + j);
      *(rrr[i] + j) = *(rrr[i] + mm);
      *(rrr[i] + mm) = temp;
      temp = *(f + i*200 + j);
      *(f + i*200 + j) = *(f + i*200 + mm);
      *(f + i*200 + mm) = temp;
      temp = *(g + i*200 + j);
      *(g + i*200 + j) = *(g + i*200 + mm);
      *(g + i*200 + mm) = temp;
    }
    
    int aaa, bbb, ccc, *ddd;
    aaa = append_line(CanvasWK, rrr[i], (f + i*200), length, rr, gg, bb);
    bbb = append_line(CanvasGK, rrr[i], (g + i*200), length, rr, gg, bb);
    ccc = append_line(CanvasGW, (f+i*200), (g + i*200), length, rr, gg, bb);
    ddd = (int *)g_malloc(sizeof(int)*3);
    *ddd = aaa;
    *(ddd+1) = bbb;
    *(ddd+2) = bbb;
    (DCURVE+i)->lines = g_list_append(NULL, (gpointer) ddd);
    (DCURVE+i)->w = f + i*200;
    (DCURVE+i)->g = g + i*200;
    (DCURVE+i)->k = rrr[i];
    (DCURVE+i)->length = length;
    (DCURVE+i)->color_r = rr;
    (DCURVE+i)->color_g = gg;
    (DCURVE+i)->color_b = bb;
    (DCURVE+i)->isVisible = TRUE;
    (DCURVE+i)->num = i;

    /// get max;
    double gmax = *(g+i*200);
    int index = 0;
    for(int ii = 1; ii<length;ii++){
      if(*(g+i*200+ii) > gmax){
        index = ii;
        gmax = *(g+i*200+ii);
      }
    }
    (DCURVE+i)->max_g = index;
  }
  gtk_widget_queue_draw(CanvasWK);
  gtk_widget_queue_draw(CanvasGK);
  gtk_widget_queue_draw(CanvasGW);
}

static void cb_main4(GtkWidget *w, gpointer user_data){
  double thr, yr, yi;
  double h_val[200*200];
  double x1[200*200], x2[200*200], y1[200*200], y2[200*200];
  update_val1();
  if(istate >= 0)override_val();
  calculate_unit();
  get_basic_parameters();

  // calculate the table of H
  thr = theta/180*M_PI;
  yr = xcs*sin(thr);
  yi = xcs*cos(thr);
  bessmd(yr);
  for(int iw = 0; iw < 200; iw++){
    for(int ig = 0; ig < 200; ig++){
      complex f;
      x1[iw*200 + ig] = (cxa - cxi)/200*iw + cxi;
      x2[iw*200 + ig] = (cxa - cxi)/200*(iw+1) + cxi;
      y1[iw*200 + ig] = (cya - cyi)/200*ig + cyi;
      y2[iw*200 + ig] = (cya - cyi)/200*(ig+1) + cyi;
      f= (cxa - cxi)/200*(iw+0.5) + cxi + I*( (cya - cyi)/200*(ig+0.5) + cyi);
      h_val[iw*200 + ig] = log10(cabs(htodh(f, yr, yi)));
    }
  }

  // create a color contour
  GtkWidget *win;
  win = main4_new(x1, y1, x2, y2, h_val, 200*200);
  gtk_widget_show_all(win);
}

static void cb_parameters(GtkWidget *w, GtkWidget *window){
  GtkWidget *dialog, *vbox,*hbox;
  GtkWidget *label;
  gint response;

  update_val1();
  if(istate >= 0)override_val();
  get_basic_parameters();
  dialog = gtk_dialog_new();
  vbox = GTK_DIALOG(dialog)->vbox;

  hbox = gtk_hbox_new(TRUE,1);
  label = gtk_label_new("");
  gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  for(int i = 0; i < num; i++){
    label = gtk_label_new(*(sname+i));
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  }
  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);

  {
    GtkWidget *hb_q, *hb_qm, *hb_n, *hb_wc, *hb_wp, *hb_vpa, *hb_vpe, *hb_dr, *hb_Tpa, *hb_Tpe;
    GtkWidget *hb_A, *hb_P;
    hb_q = gtk_hbox_new(TRUE,1);
    hb_qm = gtk_hbox_new(TRUE,1);
    hb_n = gtk_hbox_new(TRUE,1);
    hb_wc = gtk_hbox_new(TRUE,1);
    hb_wp = gtk_hbox_new(TRUE,1);
    hb_vpa = gtk_hbox_new(TRUE,1);
    hb_vpe = gtk_hbox_new(TRUE,1);
    hb_dr = gtk_hbox_new(TRUE,1);
    hb_Tpa = gtk_hbox_new(TRUE, 1);
    hb_Tpe = gtk_hbox_new(TRUE, 1);
    hb_A = gtk_hbox_new(TRUE,1);
    hb_P = gtk_hbox_new(TRUE,1);
    gtk_box_pack_start_defaults(GTK_BOX(hb_q), gtk_label_new("q"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_qm), gtk_label_new("qm"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_n), gtk_label_new("n"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_wc), gtk_label_new("wc"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_wp), gtk_label_new("wp"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_vpa), gtk_label_new("vpa"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_vpe), gtk_label_new("vpe"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_dr), gtk_label_new("dr"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_Tpa), gtk_label_new("Tpa"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_Tpe), gtk_label_new("Tpe"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_A), gtk_label_new("A"));
    gtk_box_pack_start_defaults(GTK_BOX(hb_P), gtk_label_new("P"));
    for(int i = 0; i < num; i++){
      gchar text[20];
      snprintf(text,19, "%e", *(an+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_q), gtk_label_new(text));
      snprintf(text,19, "%e", *(qm+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_qm), gtk_label_new(text));
      snprintf(text,19, "%e", *(rn+i) * pow(*wpd,2)/ *an / *qm);
      gtk_box_pack_start_defaults(GTK_BOX(hb_n), gtk_label_new(text));
      snprintf(text,19, "%e", *(qm+i) *b);
      gtk_box_pack_start_defaults(GTK_BOX(hb_wc), gtk_label_new(text));
      snprintf(text,19, "%e", *(wp+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_wp), gtk_label_new(text));
      snprintf(text,19, "%e", *(path+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_vpa), gtk_label_new(text));
      snprintf(text,19, "%e", *(peth+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_vpe), gtk_label_new(text));
      snprintf(text,19, "%e", *(spa+i));
      gtk_box_pack_start_defaults(GTK_BOX(hb_dr), gtk_label_new(text));
      {
        double Tppa, Tppe, beta, rho,AAA;
        Tppa = *(an+i)/ *(qm+i)* *(path+i)* *(path+i);
        if(*(loss0 + i) == 2 || *(loss0 + i) == 3) Tppa /= (1 - 3./2./ *(kappa+i));
        snprintf(text,19, "%e", Tppa);
        gtk_box_pack_start_defaults(GTK_BOX(hb_Tpa), gtk_label_new(text));
        Tppe = *(an+i)/ *(qm+i)* *(peth+i)* *(peth+i);
        beta = *(bfac+i);
        rho = *(ppp+i);
        if(*(loss0 + i) == 1 || *(loss0 + i) == 3) Tppe *= ((1- rho*beta*beta)/(1-rho*beta));
        snprintf(text,19, "%e", Tppe);
        gtk_box_pack_start_defaults(GTK_BOX(hb_Tpe), gtk_label_new(text));
        snprintf(text,19, "%e", Tppe/Tppa-1);
        gtk_box_pack_start_defaults(GTK_BOX(hb_A), gtk_label_new(text));
      }
      snprintf(text,19, "%e", *(wp+i)/(*(qm+i)*b));
      gtk_box_pack_start_defaults(GTK_BOX(hb_P), gtk_label_new(text));
    }
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_q);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_qm);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_n);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_wc);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_wp);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_vpa);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_vpe);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_dr);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_Tpa);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_Tpe);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_A);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hb_P);
  }

  gchar text[20];
  hbox = gtk_hbox_new(TRUE,1);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("c"));
  snprintf(text,19, "%e", cv);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(text));
  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
  hbox = gtk_hbox_new(TRUE,1);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("B"));
  snprintf(text,19, "%e", b);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(text));
  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
  hbox = gtk_hbox_new(TRUE,1);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("Val"));
  snprintf(text,19, "%e", val);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(text));
  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);

  gtk_widget_show_all(vbox);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);

}

static double t_min, t_max, t_n;
static GtkAdjustment *theta_min, *theta_max, *data_count;

static void cb_value_changed(GtkWidget *w, GtkWidget *dtheta)
{
  static char text[10];
  t_min = gtk_adjustment_get_value(theta_min);
  t_max = gtk_adjustment_get_value(theta_max);
  t_n   = gtk_adjustment_get_value(data_count);
  snprintf(text, 9, "%.3f", (t_max - t_min)/(t_n-1));
  gtk_label_set_text(GTK_LABEL(dtheta), text);

}


static void cb_calculate90(GtkWidget *w, GtkWidget *window){
  GtkWidget *dialog, *hbox;
  gint response;
  GtkWidget *spin_min, *spin_max, *spin_d, *dtheta;
  GtkWidget *entry;

  t_min = 0.;
  t_max = 90.;
  t_n = 91;

  // create spin-box
  theta_min = GTK_ADJUSTMENT( gtk_adjustment_new(0, 0, 90, 1 , 0, 0));
  theta_max = GTK_ADJUSTMENT( gtk_adjustment_new(90, 0, 90, 1, 0, 0));
  data_count   = GTK_ADJUSTMENT( gtk_adjustment_new(91, 1, 100000, 1, 0, 0));
  spin_min = gtk_spin_button_new(theta_min, 1, 3);
  spin_max = gtk_spin_button_new(theta_max, 1, 3);
  spin_d   = gtk_spin_button_new(data_count,   1, 0);

  // calculate n
  dtheta = gtk_label_new("1.000");
  g_signal_connect(G_OBJECT(theta_min), "value-changed", G_CALLBACK(cb_value_changed), (gpointer)dtheta);
  g_signal_connect(G_OBJECT(theta_max), "value-changed", G_CALLBACK(cb_value_changed), (gpointer)dtheta);
  g_signal_connect(G_OBJECT(data_count)  , "value-changed", G_CALLBACK(cb_value_changed), (gpointer)dtheta);

  entry = gtk_entry_new();

  // packing
  hbox = gtk_hbox_new(FALSE,3);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), spin_min);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), spin_max);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), spin_d);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), dtheta);

  // create dialog
  dialog = gtk_dialog_new_with_buttons("Calculate90", NULL, GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry);
  
  gtk_widget_show_all(dialog);
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  if(response == GTK_RESPONSE_OK){
    gchar *text;
    update_val1();
    if(istate != -1) override_val();
    text = gtk_entry_get_text(GTK_ENTRY(entry));
    for_all_direction(text, t_min, t_max, (int)t_n);

    // save parameters
    gchar filename[1024];
    FILE *fp;
    snprintf(filename, 1023, "%s.prma", text);
    fp = fopen(filename, "w");
    fprintf(fp, "%e %e %d\n", t_min, t_max, (int)t_n);
    fprintf(fp, "%d %e %e\n", num, wc, cv);
    fprintf(fp, "%d %d %d\n", nx, ny, nk);
    fprintf(fp, "%e\n", theta);
    fprintf(fp, "%e %e\n", xcs, xcl);
    fprintf(fp, "%e %e\n", cxi, cxa);
    fprintf(fp, "%e %e\n", cyi, cya);
    for(int i = 0;i < num; i++){
      fprintf(fp, "%s\n", *(sname + i));
      fprintf(fp, "%e %e %e\n", *(an + i), *(wpd + i), *(qm + i));
      fprintf(fp, "%e %e %e\n", *(peth + i), *(path + i), *(spa + i));
      fprintf(fp, "%d %d %e %e\n", *(loss0 + i), *(kappa+i), *(bfac + i), *(ppp + i));
    }

    fclose(fp);
  }
  /*
  dialog = gtk_file_chooser_dialog_new("Save data", GTK_WINDOW(window), GTK_FILE_CHOOSER_ACTION_SAVE,\
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
  gtk_widget_show_all(dialog);
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  if ( response == GTK_RESPONSE_ACCEPT){
    gchar *filename;
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    update_val1();
    if(istate != -1) override_val();
    for_all_direction(filename);
  }
  */
  gtk_widget_destroy(dialog);
}

static void cb_changed(GtkWidget *combo, gpointer user_data){
  gchar *text;
  text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(combo));
  if(!g_strcmp0("Add", text)){
    if(istate >= 0)override_val();
    istate = -1;
    // not implemented yet
  }else{
    gint itext;
    gchar t[25];
    if(istate >= 0)override_val();
    itext = atoi(text) - 1;
    snprintf(t,20,"%e", *(an+itext));
    gtk_entry_set_text(GTK_ENTRY(B_q), t);
    snprintf(t,20,"%e", *(qm+itext));
    gtk_entry_set_text(GTK_ENTRY(B_qm), t);
    snprintf(t,20,"%e", *(wpd+itext));
    gtk_entry_set_text(GTK_ENTRY(B_wpd), t);
    snprintf(t,20,"%e", *(spa+itext));
    gtk_entry_set_text(GTK_ENTRY(B_drift), t);
    snprintf(t,20,"%e", *(path+itext));
    gtk_entry_set_text(GTK_ENTRY(B_para), t);
    snprintf(t,20,"%e", *(peth+itext));
    gtk_entry_set_text(GTK_ENTRY(B_perp), t);
    //snprintf(t,20,"%d", *(loss0+itext));
    //gtk_entry_set_text(GTK_ENTRY(B_loss0), t);
    gtk_combo_box_set_active(GTK_COMBO_BOX(C_mode), *(loss0+itext));
    snprintf(t,20,"%d", *(kappa+itext));
    gtk_entry_set_text(GTK_ENTRY(B_kappa), t);
    snprintf(t,20,"%e", *(bfac+itext));
    gtk_entry_set_text(GTK_ENTRY(B_bfac), t);
    snprintf(t,20,"%e", *(ppp+itext));
    gtk_entry_set_text(GTK_ENTRY(B_ppp), t);
    snprintf(t,24,"%s", *(sname + itext));
    gtk_entry_set_text(GTK_ENTRY(B_name), t);
    
    istate = itext;
  }
  g_free(text);
}

#define X_NUM 401
#define Y_NUM 201
static void cb_distribution(GtkWidget *w, gpointer user_data){
  double vperp_max, vpara_min, vpara_max;
  int flags[8];

  *(flags + 0) = 0;
  *(flags + 1) = 1;
  *(flags + 2) = 0;
  *(flags + 3) = 0;
  *(flags + 4) = 0;

  update_val1();
  if(istate >= 0)override_val();
  get_basic_parameters();

  // determine range
  vperp_max = -DBL_MAX;
  vpara_min = DBL_MAX;
  vpara_max = -DBL_MAX;
  for(int i = 0; i < num; i++){
    if(*(flags+i)){
      double v_ref;
      v_ref = 2*(*(peth + i));
      vperp_max = (vperp_max > v_ref) ? vperp_max : v_ref;
      v_ref = -2*(*(path + i)) + *(spa+i);
      vpara_min = (vpara_min < v_ref) ? vpara_min : v_ref;
      v_ref =  2*(*(path + i)) + *(spa+i);
      vpara_max = (vpara_max > v_ref) ? vpara_max : v_ref;
    }
  }


  // calculate density
  double *density;
  density = (double *) calloc(X_NUM*Y_NUM, sizeof(double));
  for(int i = 0; i < num; i++){
    if(*(flags+i)){
      double vpara, vperp;
      for(int iperp = 0; iperp < Y_NUM; iperp++){
        vperp = vperp_max/(Y_NUM-1)*iperp;
        for(int ipara = 0; ipara < X_NUM; ipara++){
          double val;
          vpara = (vpara_max - vpara_min)/(X_NUM-1)*ipara + vpara_min;
          if(*(loss0 + i) == 0 || *(loss0+i) == 1)
            //val = *(rn+i) *1/ (*(path + i)*sqrt(2*M_PI)) * exp(-pow((vpara - *(spa+i))/ *(path+i),2)/2);
            val = 1/ (*(path + i)*sqrt(2*M_PI)) * exp(-pow((vpara - *(spa+i))/ *(path+i),2)/2);
          else
            //val = *(rn+i) *1/ (*(path + i)*sqrt(2*M_PI)) * tgamma(*(kappa+i))/sqrt(*(kappa+i))/tgamma(*(kappa+i) -.5)*pow(1+pow(vpara/ *(path+i),2)/2/ *(kappa+i), -*(kappa+i));
            val = 1/ (*(path + i)*sqrt(2*M_PI)) * tgamma(*(kappa+i))/sqrt(*(kappa+i))/tgamma(*(kappa+i) -.5)*pow(1+pow(vpara/ *(path+i),2)/2/ *(kappa+i), -*(kappa+i));
          if(*(loss0 + i) == 0 || *(loss0+i) == 2)
            //val *= *(rn+i) *1/M_PI/2. /pow(*(peth+i),2) * exp(-pow(vperp/ *(peth+i),2)/2);
            val *= 1/M_PI/2. /pow(*(peth+i),2) * exp(-pow(vperp/ *(peth+i),2)/2);
          else
            //val *= *(rn+i) *1/M_PI/2. /pow(*(peth+i),2) /(1- *(bfac+i)* *(ppp+i)) * (exp(-pow(vperp/ *(peth+i),2)/2) - *(ppp+i)*exp(-pow(vperp/ *(peth+i),2)/2/ *(bfac+i)));;
            val *= 1/M_PI/2. /pow(*(peth+i),2) /(1- *(bfac+i)* *(ppp+i)) * (exp(-pow(vperp/ *(peth+i),2)/2) - *(ppp+i)*exp(-pow(vperp/ *(peth+i),2)/2/ *(bfac+i)));;
          *(density + iperp*X_NUM + ipara) += val;
        }
      }
    }
  }

  for(int i = 0;i < Y_NUM; i++){
    for(int j = 0; j < X_NUM; j++)
      printf("%.10e\t", *(density + i*X_NUM + j));
    printf("\n");
  }

  free(density);
  g_message("%f, %f\n%f", vpara_min, vpara_max, vperp_max);
}

#include "menu.c"


static gboolean close_credit(GtkWidget *credit){
  gtk_dialog_response(GTK_DIALOG(credit), GTK_RESPONSE_NONE);
}

int main(int argc, char *argv[]){
  GtkWidget *window, *vbox;
  PangoFontDescription *p;
  
  p = pango_font_description_from_string("Times 18");
  gtk_init(&argc, &argv);


  // create credit
  GtkWidget *credit;
  credit =  gtk_about_dialog_new();
  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(credit), "0.1");
  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(credit), "(C) 2014 Space Group of Kyoto University");
  //gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(credit), "This is license-free");
  GdkPixbuf *image;
  GError *error = NULL;
  image = gdk_pixbuf_new_from_file("./rish-logo.jpg", &error);
  gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(credit), image);
  gtk_widget_show_all(credit);
  guint id;
  id = g_timeout_add_seconds(10, close_credit, credit);
  gtk_dialog_run(GTK_DIALOG(credit));
  g_source_remove(id);
  gtk_widget_destroy(credit);


  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "KUPDAP");
  g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(destroy), NULL);

  vbox = gtk_vbox_new(FALSE, 3);
  gtk_container_add(GTK_CONTAINER(window), vbox);

  /* create menu */
  GtkUIManager *ui;
  GtkWidget *menubar;
  ui = create_menu(window);
  g_object_set_data(G_OBJECT (window) , "ui" , (gpointer) ui);
  menubar = gtk_ui_manager_get_widget(ui , "/Menubar");
  gtk_widget_set_size_request(menubar, 300,30);
  gtk_box_pack_start_defaults(GTK_BOX(vbox) , menubar);

  init_state.an = NULL;
  init_state.loss0 = NULL;
  {
    GtkWidget *frame, *e, *hbox, *label;
    GdkColor color;
    frame = gtk_frame_new("GENERAL");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#EE4477", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), frame);
    e = gtk_event_box_new();
    gdk_color_parse("#EEAFEE", &color);
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_container_add(GTK_CONTAINER(frame), e);
    hbox = gtk_hbox_new(TRUE, 3);
    gtk_container_add(GTK_CONTAINER(e), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label),"<i>c</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox),label);
    B_cv = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_cv);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label),"&#937;<sub>1</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox),label);
    B_wc = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_wc);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#952;</i> [deg]");
    gtk_box_pack_start_defaults(GTK_BOX(hbox),label);
    B_theta = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_theta);
  }

  {
    GtkWidget *frame, *e, *vvbox, *hbox, *label;
    GdkColor color;

    /*
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#4477EE", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    */

    hbox = gtk_hbox_new(TRUE, 1);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
    label = gtk_label_new("SPECIES");
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#4477EE", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    Combo = gtk_combo_box_new_text();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), Combo);
    gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "1");
    num = 1;
    gtk_combo_box_set_active(GTK_COMBO_BOX(Combo), 0);
    gtk_combo_box_append_text(GTK_COMBO_BOX(Combo), "Add");
    id_changed = g_signal_connect(G_OBJECT(Combo), "changed", G_CALLBACK(cb_changed), NULL);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(""));//
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(""));//

    e = gtk_event_box_new();
    gdk_color_parse("#AFEEEE", &color);
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), e);
    vvbox = gtk_vbox_new(FALSE, 3);
    gtk_container_add(GTK_CONTAINER(e), vvbox);
    {
      hbox = gtk_hbox_new(TRUE, 3);
      gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
      gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("label"));
      B_name = gtk_entry_new();
      gtk_box_pack_start_defaults(GTK_BOX(hbox), B_name);
      gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new("distribution type"));
      C_mode = gtk_combo_box_new_text();
      gtk_box_pack_start_defaults(GTK_BOX(hbox), C_mode);
      gtk_combo_box_append_text(GTK_COMBO_BOX(C_mode), "bi-Maxwellian");
      gtk_combo_box_append_text(GTK_COMBO_BOX(C_mode), "subtracted Maxwellian");
      gtk_combo_box_append_text(GTK_COMBO_BOX(C_mode), "kappa-Maxwellian");
      gtk_combo_box_append_text(GTK_COMBO_BOX(C_mode), "subtracted kappa-Maxwellian");
      gtk_combo_box_set_active(GTK_COMBO_BOX(C_mode), 0);
    }

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>q</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_q = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_q);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>q/m</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_qm = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_qm);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "&#928; or -<i>N</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_wpd = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_wpd);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>v</i><sub>drift</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_drift = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_drift);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>v</i><sub>&#8869;</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_perp = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_perp);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>v</i><sub>||</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_para = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_para);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#954;</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_kappa = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_kappa);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(""));//dummy
    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(""));//dummy

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>b</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_bfac = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_bfac);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#961;</i>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_ppp = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_ppp);
  }
  {
    GtkWidget *frame, *e, *vvbox, *hbox, *label;
    GdkColor color;
    frame = gtk_frame_new("RANGE");
    label = gtk_frame_get_label_widget(GTK_FRAME(frame));
    gtk_widget_modify_font(label, p);
    gdk_color_parse("#22FB22", &color);
    gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), frame);
    e = gtk_event_box_new();
    gdk_color_parse("#98FB98", &color);
    gtk_widget_modify_bg(e, GTK_STATE_NORMAL, &color);
    gtk_container_add(GTK_CONTAINER(frame), e);
    
    vvbox = gtk_vbox_new(FALSE, 3);
    gtk_container_add(GTK_CONTAINER(e), vvbox);
    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#969;</i><sub><i>r</i> min</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_frmin = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_frmin);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#969;</i><sub><i>r</i> max</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_frmax = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_frmax);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>N</i><sub><i>&#969;r</i></sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_nfr = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_nfr);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#969;</i><sub><i>i</i> min</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_fimin = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_fimin);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>&#969;</i><sub><i>i</i> max</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_fimax = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_fimax);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>N</i><sub><i>&#969;i</i></sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_nfi = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_nfi);

    hbox = gtk_hbox_new(TRUE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vvbox), hbox);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>k</i><sub>min</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_kmin = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_kmin);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>k</i><sub>max</sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_kmax = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_kmax);
    label = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(label), "<i>N</i><sub><i>k</i></sub>");
    gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
    B_nk = gtk_entry_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), B_nk);
  }
  {
    GtkWidget *hbox, *button;
    hbox = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
    button = gtk_button_new_with_label("Plot dispersion curves");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_main2), window);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), button);
    button = gtk_button_new_with_label("dummy");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_calculate90), window);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), button);
    GtkWidget *label;
    label = gtk_bin_get_child(GTK_BIN(button));
    gtk_label_set_markup(GTK_LABEL(label),"Create data 0<sup>o</sup> - 90<sup>o</sup>");
    /*
    button = gtk_button_new_with_label("CalculateH");
    g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(cb_main4), window);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), button);
    */
  }

  sname = (char **) malloc(sizeof(char *) * num);
  *(sname) = (char *) malloc(sizeof(char) * 21);

  // initialize main2
  gtk_widget_show_all(window);
  gtk_main();
}
