#include "graph.h"
#include <gtdispc.h> 
#include "state.h"
#include <complex.h>
#define __USE_BSD
#include <math.h>

GtkWidget *MAIN3 = NULL;
GtkWidget *wlabel;
GtkWidget *klabel;
static GtkWidget *canvas_xy, *canvas_yz, *canvas_zx;
static complex Ex, Ey, Ez;
static double Px, Py, Pz;

typedef struct{
  complex x;
  complex y;
  complex z;
}c_vec;

static c_vec cross_product(c_vec A, c_vec B){
  c_vec C;
  C.x = A.y*B.z - B.y*A.z;
  C.y = A.z*B.x - B.z*A.x;
  C.z = A.x*B.y - B.x*A.y;
  return C;
}

void main3_set_E(complex ex, complex ey, complex ez, complex w){
  double norm;
  norm = sqrt(cabs(ex) + cabs(ey) + cabs(ez));
  Ex = ex/norm;
  Ey = ey/norm;
  Ez = ez/norm;

  c_vec K, B, P;
  double angle;

  if(cabs(w) == 0){
    P.x = 0; P.y = 0; P.z = 0;
  }else{
    double norm;
    angle = theta/180*M_PI;
    K.x = sin(angle);
    K.y = 0;
    K.z = cos(angle);
    c_vec E;
    E.x = Ex; E.y = Ey; E.z = Ez;
    B = cross_product(K,E);
    B.x /= w;
    B.y /= w;
    B.z /= w;
    B.x = conj(B.x);
    B.y = conj(B.y);
    B.z = conj(B.z);
    P = cross_product(E,B);
    P.x = creal(P.x);
    P.y = creal(P.y);
    P.z = creal(P.z);
    norm = sqrt(cabs(P.x*P.x + P.y*P.y + P.z*P.z));
    norm = creal(w) > 0 ? norm : -norm;
    P.x /= norm;
    P.y /= norm;
    P.z /= norm;
  }
  Px = P.x;
  Py = P.y;
  Pz = P.z;
}


static void cb_destroy(GtkWidget *w, gpointer p){
  gtk_widget_destroy(w);
  MAIN3 = NULL;
}

static gboolean cb_expose_event_xy(GtkWidget *canvas, GdkEventExpose *event, gpointer *data){
  double x[200];
  double y[200];
  for(int i = 0; i < 200; i++) x[i] = creal(Ex*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
  for(int i = 0; i < 200; i++) y[i] = creal(Ey*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
;
  GdkWindow *drawable = canvas->window;
  cairo_t *cr;
  gint width, height;
  cr = gdk_cairo_create(drawable);
  cairo_set_line_width(cr, 3);
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);

  int length;
  length = width > height ? height : width;
  length -= 5;

  // draw axis
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_move_to(cr, width/2, height/2-length/2);
  cairo_line_to(cr, width/2, height/2+length/2);
  cairo_move_to(cr, width/2 - length/2, height/2);
  cairo_line_to(cr, width/2 + length/2, height/2);
  cairo_stroke(cr);
  cairo_set_font_size(cr, 10);
  cairo_move_to(cr, width/2+5, height/2-length/2+5);
  cairo_show_text(cr, "y");
  cairo_move_to(cr, width/2 + length/2-5, height/2-5);
  cairo_show_text(cr, "x");
  cairo_stroke(cr);

  // draw polarization
  cairo_set_source_rgb(cr, 0, 1, 0);
  for(int i = 0;i<200;i++){
    cairo_line_to(cr, length/2*x[i]+width/2, height - (length/2*y[i]+height/2));
  }
  cairo_stroke(cr);

  // draw Poynting vector
  double thr,norm;
  thr = atan2(Py, Px);
  norm = sqrt(Py*Py + Px*Px);
  cairo_set_source_rgb(cr, 0, 0, 1);
  cairo_move_to(cr, width/2, height/2);
  cairo_line_to(cr, width/2 + cos(thr)*norm*length/2, height/2 - sin(thr)*norm*length/2);
  cairo_move_to(cr, width/2 + cos(thr)*norm*length/2+5, height/2 - sin(thr)*norm*length/2+5);
  cairo_show_text(cr, "P");
  cairo_stroke(cr);

  cairo_destroy(cr);
  return FALSE;
}

static gboolean cb_expose_event_yz(GtkWidget *canvas, GdkEventExpose *event, gpointer *data){
  double x[200];
  double y[200];
  for(int i = 0; i < 200; i++) x[i] = creal(Ey*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
  for(int i = 0; i < 200; i++) y[i] = creal(Ez*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
;
  GdkWindow *drawable = canvas->window;
  cairo_t *cr;
  gint width, height;
  cr = gdk_cairo_create(drawable);
  cairo_set_line_width(cr, 3);
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);

  int length;
  length = width > height ? height : width;
  length -= -5;

  // draw axis
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_move_to(cr, width/2, height/2-length/2);
  cairo_line_to(cr, width/2, height/2+length/2);
  cairo_move_to(cr, width/2 - length/2, height/2);
  cairo_line_to(cr, width/2 + length/2, height/2);
  cairo_stroke(cr);
  cairo_set_font_size(cr, 10);
  cairo_move_to(cr, width/2+5, height/2-length/2+5);
  cairo_show_text(cr, "z");
  cairo_move_to(cr, width/2 + length/2-5, height/2-5);
  cairo_show_text(cr, "y");
  cairo_stroke(cr);

  // draw polarization
  cairo_set_source_rgb(cr, 0, 1, 0);
  for(int i = 0;i<200;i++){
    cairo_line_to(cr, length/2*x[i]+width/2, height - (length/2*y[i]+height/2));
  }
  cairo_stroke(cr);

  // draw Poynting vector
  double thr,norm;
  thr = atan2(Pz, Py);
  norm = sqrt(Py*Py + Pz*Pz);
  cairo_set_source_rgb(cr, 0, 0, 1);
  cairo_move_to(cr, width/2, height/2);
  cairo_line_to(cr, width/2 + cos(thr)*norm*length/2, height/2 - sin(thr)*norm*length/2);
  cairo_move_to(cr, width/2 + cos(thr)*norm*length/2+5, height/2 - sin(thr)*norm*length/2+5);
  cairo_show_text(cr, "P");
  cairo_stroke(cr);

  cairo_destroy(cr);
  return FALSE;
}

static gboolean cb_expose_event_zx(GtkWidget *canvas, GdkEventExpose *event, gpointer *data){
  double x[200];
  double y[200];
  for(int i = 0; i < 200; i++) x[i] = creal(Ez*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
  for(int i = 0; i < 200; i++) y[i] = creal(Ex*(cos(-2*M_PI*i/199) + I*sin(-2*M_PI*i/199)));
;
  GdkWindow *drawable = canvas->window;
  cairo_t *cr;
  gint width, height;
  cr = gdk_cairo_create(drawable);
  cairo_set_line_width(cr, 3);
  gdk_window_get_geometry(drawable, NULL, NULL, &width, &height, NULL);

  int length;
  length = width > height ? height : width;
  length -= 5;

  // draw axis
  cairo_set_source_rgb(cr, 0, 0, 0);
  cairo_move_to(cr, width/2, height/2-length/2);
  cairo_line_to(cr, width/2, height/2+length/2);
  cairo_move_to(cr, width/2 - length/2, height/2);
  cairo_line_to(cr, width/2 + length/2, height/2);
  cairo_stroke(cr);
  cairo_set_font_size(cr, 10);
  cairo_move_to(cr, width/2+5, height/2-length/2+5);
  cairo_show_text(cr, "x");
  cairo_move_to(cr, width/2 + length/2-5, height/2-5);
  cairo_show_text(cr, "z");
  cairo_stroke(cr);

  // draw polarization
  cairo_set_source_rgb(cr, 0, 1, 0);
  for(int i = 0;i<200;i++){
    cairo_line_to(cr, length/2*x[i]+width/2, height - (length/2*y[i]+height/2));
  }
  cairo_stroke(cr);

  // draw k vector
  cairo_set_source_rgb(cr, 1, 0, 0);
  cairo_move_to(cr, width/2, height/2);
  cairo_line_to(cr, width/2 + cos(theta/180*M_PI)*length/2, height/2 - sin(theta/180*M_PI)*length/2);
  cairo_move_to(cr, width/2 + cos(theta/180*M_PI)*length/2+5, height/2 - sin(theta/180*M_PI)*length/2+5);
  cairo_show_text(cr, "k");
  cairo_stroke(cr);

  // draw Poynting vector
  double thr,norm;
  thr = atan2(Px, Pz);
  norm = sqrt(Px*Px + Pz*Pz);
  cairo_set_source_rgb(cr, 0, 0, 1);
  cairo_move_to(cr, width/2, height/2);
  cairo_line_to(cr, width/2 + cos(thr)*norm*length/2, height/2 - sin(thr)*norm*length/2);
  cairo_move_to(cr, width/2 + cos(thr)*norm*length/2+5, height/2 - sin(thr)*norm*length/2+5);
  cairo_show_text(cr, "P");
  cairo_stroke(cr);

  cairo_destroy(cr);
  return FALSE;
}

GtkWidget * main3_new(void){
  GtkWidget *w, *vbox, *hbox, *vvbox;;
  w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(w), FALSE);
  g_signal_connect(G_OBJECT(w), "destroy", G_CALLBACK(cb_destroy), NULL);
  
  vbox = gtk_vbox_new(TRUE, 3);
  {
    hbox = gtk_hbox_new(TRUE, 3);
    canvas_xy = gtk_drawing_area_new();
    canvas_yz = gtk_drawing_area_new();
    gtk_box_pack_start_defaults(GTK_BOX(hbox), canvas_xy);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), canvas_yz);
    g_signal_connect(G_OBJECT(canvas_xy), "expose-event", G_CALLBACK(cb_expose_event_xy), NULL);
    g_signal_connect(G_OBJECT(canvas_yz), "expose-event", G_CALLBACK(cb_expose_event_yz), NULL);
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);

    hbox = gtk_hbox_new(TRUE, 3);
    canvas_zx = gtk_drawing_area_new();
    g_signal_connect(G_OBJECT(canvas_zx), "expose-event", G_CALLBACK(cb_expose_event_zx), NULL);
    gtk_box_pack_start_defaults(GTK_BOX(hbox), canvas_zx);
    {
      vvbox = gtk_vbox_new(TRUE, 3);
      wlabel = gtk_label_new("");
      klabel = gtk_label_new("");
      gtk_box_pack_start_defaults(GTK_BOX(vvbox), wlabel);
      gtk_box_pack_start_defaults(GTK_BOX(vvbox), klabel);
      gtk_box_pack_start_defaults(GTK_BOX(hbox), vvbox);
    }
    gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
  }
  gtk_container_add(GTK_CONTAINER(w), vbox);


  return w;
}

